# The following is only needed because of Python 3.7
# mypy: no-warn-unused-ignores
from __future__ import annotations

import calendar
import math

from datetime import date
from datetime import datetime
from datetime import timedelta
from typing import TYPE_CHECKING
from typing import ClassVar
from typing import NoReturn
from typing import cast
from typing import overload

import pendulum

from pendulum.constants import MONTHS_PER_YEAR
from pendulum.constants import YEARS_PER_CENTURY
from pendulum.constants import YEARS_PER_DECADE
from pendulum.day import WeekDay
from pendulum.exceptions import PendulumException
from pendulum.helpers import add_duration
from pendulum.interval import Interval
from pendulum.mixins.default import FormattableMixin


if TYPE_CHECKING:
    from typing_extensions import Self
    from typing_extensions import SupportsIndex


class Date(FormattableMixin, date):
    _MODIFIERS_VALID_UNITS: ClassVar[list[str]] = [
        "day",
        "week",
        "month",
        "year",
        "decade",
        "century",
    ]

    # Getters/Setters

    def set(
        self, year: int | None = None, month: int | None = None, day: int | None = None
    ) -> Self:
        return self.replace(year=year, month=month, day=day)

    @property
    def day_of_week(self) -> WeekDay:
        """
        Returns the day of the week (0-6).
        """
        return WeekDay(self.weekday())

    @property
    def day_of_year(self) -> int:
        """
        Returns the day of the year (1-366).
        """
        k = 1 if self.is_leap_year() else 2

        return (275 * self.month) // 9 - k * ((self.month + 9) // 12) + self.day - 30

    @property
    def week_of_year(self) -> int:
        return self.isocalendar()[1]

    @property
    def days_in_month(self) -> int:
        return calendar.monthrange(self.year, self.month)[1]

    @property
    def week_of_month(self) -> int:
        return math.ceil((self.day + self.first_of("month").isoweekday() - 1) / 7)

    @property
    def age(self) -> int:
        return self.diff(abs=False).in_years()

    @property
    def quarter(self) -> int:
        return math.ceil(self.month / 3)

    # String Formatting

    def to_date_string(self) -> str:
        """
        Format the instance as date.

        :rtype: str
        """
        return self.strftime("%Y-%m-%d")

    def to_formatted_date_string(self) -> str:
        """
        Format the instance as a readable date.

        :rtype: str
        """
        return self.strftime("%b %d, %Y")

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}({self.year}, {self.month}, {self.day})"

    # COMPARISONS

    def closest(self, dt1: date, dt2: date) -> Self:
        """
        Get the closest date from the instance.
        """
        dt1 = self.__class__(dt1.year, dt1.month, dt1.day)
        dt2 = self.__class__(dt2.year, dt2.month, dt2.day)

        if self.diff(dt1).in_seconds() < self.diff(dt2).in_seconds():
            return dt1

        return dt2

    def farthest(self, dt1: date, dt2: date) -> Self:
        """
        Get the farthest date from the instance.
        """
        dt1 = self.__class__(dt1.year, dt1.month, dt1.day)
        dt2 = self.__class__(dt2.year, dt2.month, dt2.day)

        if self.diff(dt1).in_seconds() > self.diff(dt2).in_seconds():
            return dt1

        return dt2

    def is_future(self) -> bool:
        """
        Determines if the instance is in the future, ie. greater than now.
        """
        return self > self.today()

    def is_past(self) -> bool:
        """
        Determines if the instance is in the past, ie. less than now.
        """
        return self < self.today()

    def is_leap_year(self) -> bool:
        """
        Determines if the instance is a leap year.
        """
        return calendar.isleap(self.year)

    def is_long_year(self) -> bool:
        """
        Determines if the instance is a long year

        See link `<https://en.wikipedia.org/wiki/ISO_8601#Week_dates>`_
        """
        return Date(self.year, 12, 28).isocalendar()[1] == 53

    def is_same_day(self, dt: date) -> bool:
        """
        Checks if the passed in date is the same day as the instance current day.
        """
        return self == dt

    def is_anniversary(self, dt: date | None = None) -> bool:
        """
        Check if it's the anniversary.

        Compares the date/month values of the two dates.
        """
        if dt is None:
            dt = self.__class__.today()

        instance = self.__class__(dt.year, dt.month, dt.day)

        return (self.month, self.day) == (instance.month, instance.day)

    # the additional method for checking if today is the anniversary day
    # the alias is provided to start using a new name and keep the backward
    # compatibility the old name can be completely replaced with the new in
    # one of the future versions
    is_birthday = is_anniversary

    # ADDITIONS AND SUBTRACTIONS

    def add(
        self, years: int = 0, months: int = 0, weeks: int = 0, days: int = 0
    ) -> Self:
        """
        Add duration to the instance.

        :param years: The number of years
        :param months: The number of months
        :param weeks: The number of weeks
        :param days: The number of days
        """
        dt = add_duration(
            date(self.year, self.month, self.day),
            years=years,
            months=months,
            weeks=weeks,
            days=days,
        )

        return self.__class__(dt.year, dt.month, dt.day)

    def subtract(
        self, years: int = 0, months: int = 0, weeks: int = 0, days: int = 0
    ) -> Self:
        """
        Remove duration from the instance.

        :param years: The number of years
        :param months: The number of months
        :param weeks: The number of weeks
        :param days: The number of days
        """
        return self.add(years=-years, months=-months, weeks=-weeks, days=-days)

    def _add_timedelta(self, delta: timedelta) -> Self:
        """
        Add timedelta duration to the instance.

        :param delta: The timedelta instance
        """
        if isinstance(delta, pendulum.Duration):
            return self.add(
                years=delta.years,
                months=delta.months,
                weeks=delta.weeks,
                days=delta.remaining_days,
            )

        return self.add(days=delta.days)

    def _subtract_timedelta(self, delta: timedelta) -> Self:
        """
        Remove timedelta duration from the instance.

        :param delta: The timedelta instance
        """
        if isinstance(delta, pendulum.Duration):
            return self.subtract(
                years=delta.years,
                months=delta.months,
                weeks=delta.weeks,
                days=delta.remaining_days,
            )

        return self.subtract(days=delta.days)

    def __add__(self, other: timedelta) -> Self:
        if not isinstance(other, timedelta):
            return NotImplemented

        return self._add_timedelta(other)

    @overload  # type: ignore[override]  # this is only needed because of Python 3.7
    def __sub__(self, __delta: timedelta) -> Self: ...

    @overload
    def __sub__(self, __dt: datetime) -> NoReturn: ...

    @overload
    def __sub__(self, __dt: Self) -> Interval[Date]: ...

    def __sub__(self, other: timedelta | date) -> Self | Interval[Date]:
        if isinstance(other, timedelta):
            return self._subtract_timedelta(other)

        if not isinstance(other, date):
            return NotImplemented

        dt = self.__class__(other.year, other.month, other.day)

        return dt.diff(self, False)

    # DIFFERENCES

    def diff(self, dt: date | None = None, abs: bool = True) -> Interval[Date]:
        """
        Returns the difference between two Date objects as an Interval.

        :param dt: The date to compare to (defaults to today)
        :param abs: Whether to return an absolute interval or not
        """
        if dt is None:
            dt = self.today()

        return Interval(self, Date(dt.year, dt.month, dt.day), absolute=abs)

    def diff_for_humans(
        self,
        other: date | None = None,
        absolute: bool = False,
        locale: str | None = None,
    ) -> str:
        """
        Get the difference in a human readable format in the current locale.

        When comparing a value in the past to default now:
        1 day ago
        5 months ago

        When comparing a value in the future to default now:
        1 day from now
        5 months from now

        When comparing a value in the past to another value:
        1 day before
        5 months before

        When comparing a value in the future to another value:
        1 day after
        5 months after

        :param other: The date to compare to (defaults to today)
        :param absolute: removes time difference modifiers ago, after, etc
        :param locale: The locale to use for localization
        """
        is_now = other is None

        if is_now:
            other = self.today()

        diff = self.diff(other)

        return pendulum.format_diff(diff, is_now, absolute, locale)

    # MODIFIERS

    def start_of(self, unit: str) -> Self:
        """
        Returns a copy of the instance with the time reset
        with the following rules:

        * day: time to 00:00:00
        * week: date to first day of the week and time to 00:00:00
        * month: date to first day of the month and time to 00:00:00
        * year: date to first day of the year and time to 00:00:00
        * decade: date to first day of the decade and time to 00:00:00
        * century: date to first day of century and time to 00:00:00

        :param unit: The unit to reset to
        """
        if unit not in self._MODIFIERS_VALID_UNITS:
            raise ValueError(f'Invalid unit "{unit}" for start_of()')

        return cast("Self", getattr(self, f"_start_of_{unit}")())

    def end_of(self, unit: str) -> Self:
        """
        Returns a copy of the instance with the time reset
        with the following rules:

        * week: date to last day of the week
        * month: date to last day of the month
        * year: date to last day of the year
        * decade: date to last day of the decade
        * century: date to last day of century

        :param unit: The unit to reset to
        """
        if unit not in self._MODIFIERS_VALID_UNITS:
            raise ValueError(f'Invalid unit "{unit}" for end_of()')

        return cast("Self", getattr(self, f"_end_of_{unit}")())

    def _start_of_day(self) -> Self:
        """
        Compatibility method.
        """
        return self

    def _end_of_day(self) -> Self:
        """
        Compatibility method
        """
        return self

    def _start_of_month(self) -> Self:
        """
        Reset the date to the first day of the month.
        """
        return self.set(self.year, self.month, 1)

    def _end_of_month(self) -> Self:
        """
        Reset the date to the last day of the month.
        """
        return self.set(self.year, self.month, self.days_in_month)

    def _start_of_year(self) -> Self:
        """
        Reset the date to the first day of the year.
        """
        return self.set(self.year, 1, 1)

    def _end_of_year(self) -> Self:
        """
        Reset the date to the last day of the year.
        """
        return self.set(self.year, 12, 31)

    def _start_of_decade(self) -> Self:
        """
        Reset the date to the first day of the decade.
        """
        year = self.year - self.year % YEARS_PER_DECADE

        return self.set(year, 1, 1)

    def _end_of_decade(self) -> Self:
        """
        Reset the date to the last day of the decade.
        """
        year = self.year - self.year % YEARS_PER_DECADE + YEARS_PER_DECADE - 1

        return self.set(year, 12, 31)

    def _start_of_century(self) -> Self:
        """
        Reset the date to the first day of the century.
        """
        year = self.year - 1 - (self.year - 1) % YEARS_PER_CENTURY + 1

        return self.set(year, 1, 1)

    def _end_of_century(self) -> Self:
        """
        Reset the date to the last day of the century.
        """
        year = self.year - 1 - (self.year - 1) % YEARS_PER_CENTURY + YEARS_PER_CENTURY

        return self.set(year, 12, 31)

    def _start_of_week(self) -> Self:
        """
        Reset the date to the first day of the week.
        """
        dt = self

        if self.day_of_week != pendulum._WEEK_STARTS_AT:
            dt = self.previous(pendulum._WEEK_STARTS_AT)

        return dt.start_of("day")

    def _end_of_week(self) -> Self:
        """
        Reset the date to the last day of the week.
        """
        dt = self

        if self.day_of_week != pendulum._WEEK_ENDS_AT:
            dt = self.next(pendulum._WEEK_ENDS_AT)

        return dt.end_of("day")

    def next(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the next occurrence of a given day of the week.
        If no day_of_week is provided, modify to the next occurrence
        of the current day of the week.  Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.

        :param day_of_week: The next day of week to reset to.
        """
        if day_of_week is None:
            day_of_week = self.day_of_week

        if day_of_week < WeekDay.MONDAY or day_of_week > WeekDay.SUNDAY:
            raise ValueError("Invalid day of week")

        dt = self.add(days=1)
        while dt.day_of_week != day_of_week:
            dt = dt.add(days=1)

        return dt

    def previous(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the previous occurrence of a given day of the week.
        If no day_of_week is provided, modify to the previous occurrence
        of the current day of the week.  Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.

        :param day_of_week: The previous day of week to reset to.
        """
        if day_of_week is None:
            day_of_week = self.day_of_week

        if day_of_week < WeekDay.MONDAY or day_of_week > WeekDay.SUNDAY:
            raise ValueError("Invalid day of week")

        dt = self.subtract(days=1)
        while dt.day_of_week != day_of_week:
            dt = dt.subtract(days=1)

        return dt

    def first_of(self, unit: str, day_of_week: WeekDay | None = None) -> Self:
        """
        Returns an instance set to the first occurrence
        of a given day of the week in the current unit.
        If no day_of_week is provided, modify to the first day of the unit.
        Use the supplied consts to indicate the desired day_of_week,
        ex. pendulum.MONDAY.

        Supported units are month, quarter and year.

        :param unit: The unit to use
        :param day_of_week: The day of week to reset to.
        """
        if unit not in ["month", "quarter", "year"]:
            raise ValueError(f'Invalid unit "{unit}" for first_of()')

        return cast("Self", getattr(self, f"_first_of_{unit}")(day_of_week))

    def last_of(self, unit: str, day_of_week: WeekDay | None = None) -> Self:
        """
        Returns an instance set to the last occurrence
        of a given day of the week in the current unit.
        If no day_of_week is provided, modify to the last day of the unit.
        Use the supplied consts to indicate the desired day_of_week,
        ex. pendulum.MONDAY.

        Supported units are month, quarter and year.

        :param unit: The unit to use
        :param day_of_week: The day of week to reset to.
        """
        if unit not in ["month", "quarter", "year"]:
            raise ValueError(f'Invalid unit "{unit}" for first_of()')

        return cast("Self", getattr(self, f"_last_of_{unit}")(day_of_week))

    def nth_of(self, unit: str, nth: int, day_of_week: WeekDay) -> Self:
        """
        Returns a new instance set to the given occurrence
        of a given day of the week in the current unit.
        If the calculated occurrence is outside the scope of the current unit,
        then raise an error. Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.

        Supported units are month, quarter and year.

        :param unit: The unit to use
        :param nth: The occurrence to use
        :param day_of_week: The day of week to set to.
        """
        if unit not in ["month", "quarter", "year"]:
            raise ValueError(f'Invalid unit "{unit}" for first_of()')

        dt = cast("Self", getattr(self, f"_nth_of_{unit}")(nth, day_of_week))
        if not dt:
            raise PendulumException(
                f"Unable to find occurrence {nth}"
                f" of {WeekDay(day_of_week).name.capitalize()} in {unit}"
            )

        return dt

    def _first_of_month(self, day_of_week: WeekDay) -> Self:
        """
        Modify to the first occurrence of a given day of the week
        in the current month. If no day_of_week is provided,
        modify to the first day of the month. Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.

        :param day_of_week: The day of week to set to.
        """
        dt = self

        if day_of_week is None:
            return dt.set(day=1)

        month = calendar.monthcalendar(dt.year, dt.month)

        calendar_day = day_of_week

        if month[0][calendar_day] > 0:
            day_of_month = month[0][calendar_day]
        else:
            day_of_month = month[1][calendar_day]

        return dt.set(day=day_of_month)

    def _last_of_month(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the last occurrence of a given day of the week
        in the current month. If no day_of_week is provided,
        modify to the last day of the month. Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.

        :param day_of_week: The day of week to set to.
        """
        dt = self

        if day_of_week is None:
            return dt.set(day=self.days_in_month)

        month = calendar.monthcalendar(dt.year, dt.month)

        calendar_day = day_of_week

        if month[-1][calendar_day] > 0:
            day_of_month = month[-1][calendar_day]
        else:
            day_of_month = month[-2][calendar_day]

        return dt.set(day=day_of_month)

    def _nth_of_month(self, nth: int, day_of_week: WeekDay) -> Self | None:
        """
        Modify to the given occurrence of a given day of the week
        in the current month. If the calculated occurrence is outside,
        the scope of the current month, then return False and no
        modifications are made. Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.
        """
        if nth == 1:
            return self.first_of("month", day_of_week)

        dt = self.first_of("month")
        check = dt.format("YYYY-MM")
        for _ in range(nth - (1 if dt.day_of_week == day_of_week else 0)):
            dt = dt.next(day_of_week)

        if dt.format("YYYY-MM") == check:
            return self.set(day=dt.day)

        return None

    def _first_of_quarter(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the first occurrence of a given day of the week
        in the current quarter. If no day_of_week is provided,
        modify to the first day of the quarter. Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.
        """
        return self.set(self.year, self.quarter * 3 - 2, 1).first_of(
            "month", day_of_week
        )

    def _last_of_quarter(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the last occurrence of a given day of the week
        in the current quarter. If no day_of_week is provided,
        modify to the last day of the quarter. Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.
        """
        return self.set(self.year, self.quarter * 3, 1).last_of("month", day_of_week)

    def _nth_of_quarter(self, nth: int, day_of_week: WeekDay) -> Self | None:
        """
        Modify to the given occurrence of a given day of the week
        in the current quarter. If the calculated occurrence is outside,
        the scope of the current quarter, then return False and no
        modifications are made. Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.
        """
        if nth == 1:
            return self.first_of("quarter", day_of_week)

        dt = self.replace(self.year, self.quarter * 3, 1)
        last_month = dt.month
        year = dt.year
        dt = dt.first_of("quarter")
        for _ in range(nth - (1 if dt.day_of_week == day_of_week else 0)):
            dt = dt.next(day_of_week)

        if last_month < dt.month or year != dt.year:
            return None

        return self.set(self.year, dt.month, dt.day)

    def _first_of_year(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the first occurrence of a given day of the week
        in the current year. If no day_of_week is provided,
        modify to the first day of the year. Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.
        """
        return self.set(month=1).first_of("month", day_of_week)

    def _last_of_year(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the last occurrence of a given day of the week
        in the current year. If no day_of_week is provided,
        modify to the last day of the year. Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.
        """
        return self.set(month=MONTHS_PER_YEAR).last_of("month", day_of_week)

    def _nth_of_year(self, nth: int, day_of_week: WeekDay) -> Self | None:
        """
        Modify to the given occurrence of a given day of the week
        in the current year. If the calculated occurrence is outside,
        the scope of the current year, then return False and no
        modifications are made. Use the supplied consts
        to indicate the desired day_of_week, ex. pendulum.MONDAY.
        """
        if nth == 1:
            return self.first_of("year", day_of_week)

        dt = self.first_of("year")
        year = dt.year
        for _ in range(nth - (1 if dt.day_of_week == day_of_week else 0)):
            dt = dt.next(day_of_week)

        if year != dt.year:
            return None

        return self.set(self.year, dt.month, dt.day)

    def average(self, dt: date | None = None) -> Self:
        """
        Modify the current instance to the average
        of a given instance (default now) and the current instance.
        """
        if dt is None:
            dt = Date.today()

        return self.add(days=int(self.diff(dt, False).in_days() / 2))

    # Native methods override

    @classmethod
    def today(cls) -> Self:
        dt = date.today()

        return cls(dt.year, dt.month, dt.day)

    @classmethod
    def fromtimestamp(cls, t: float) -> Self:
        dt = super().fromtimestamp(t)

        return cls(dt.year, dt.month, dt.day)

    @classmethod
    def fromordinal(cls, n: int) -> Self:
        dt = super().fromordinal(n)

        return cls(dt.year, dt.month, dt.day)

    def replace(
        self,
        year: SupportsIndex | None = None,
        month: SupportsIndex | None = None,
        day: SupportsIndex | None = None,
    ) -> Self:
        year = year if year is not None else self.year
        month = month if month is not None else self.month
        day = day if day is not None else self.day

        return self.__class__(year, month, day)



from __future__ import annotations

from pendulum import Time


def test_to_string():
    d = Time(1, 2, 3)
    assert str(d) == "01:02:03"
    d = Time(1, 2, 3, 123456)
    assert str(d) == "01:02:03.123456"


def test_repr():
    d = Time(1, 2, 3)
    assert repr(d) == "Time(1, 2, 3)"

    d = Time(1, 2, 3, 123456)
    assert repr(d) == "Time(1, 2, 3, 123456)"


def test_format_with_locale():
    d = Time(14, 15, 16)
    assert d.format("hh:mm:ss A", locale="fr") == "02:15:16 PM"


def test_strftime():
    d = Time(14, 15, 16)
    assert d.strftime("%H") == "14"


def test_for_json():
    d = Time(14, 15, 16)
    assert d.for_json() == "14:15:16"


def test_format():
    d = Time(14, 15, 16)
    assert f"{d}" == "14:15:16"
    assert f"{d:mm}" == "15"



from __future__ import annotations

import pendulum


def test_in_weeks():
    it = pendulum.duration(days=17)
    assert round(it.total_weeks(), 2) == 2.43


def test_in_days():
    it = pendulum.duration(days=3)
    assert it.total_days() == 3


def test_in_hours():
    it = pendulum.duration(days=3, minutes=72)
    assert it.total_hours() == 73.2


def test_in_minutes():
    it = pendulum.duration(minutes=6, seconds=72)
    assert it.total_minutes() == 7.2


def test_in_seconds():
    it = pendulum.duration(seconds=72, microseconds=123456)
    assert it.total_seconds() == 72.123456



from __future__ import annotations

import datetime

from datetime import time
from datetime import timedelta
from typing import TYPE_CHECKING
from typing import Optional
from typing import cast
from typing import overload

import pendulum

from pendulum.constants import SECS_PER_HOUR
from pendulum.constants import SECS_PER_MIN
from pendulum.constants import USECS_PER_SEC
from pendulum.duration import AbsoluteDuration
from pendulum.duration import Duration
from pendulum.mixins.default import FormattableMixin
from pendulum.tz.timezone import UTC


if TYPE_CHECKING:
    from typing_extensions import Literal
    from typing_extensions import Self
    from typing_extensions import SupportsIndex

    from pendulum.tz.timezone import FixedTimezone
    from pendulum.tz.timezone import Timezone


class Time(FormattableMixin, time):
    """
    Represents a time instance as hour, minute, second, microsecond.
    """

    @classmethod
    def instance(
        cls, t: time, tz: str | Timezone | FixedTimezone | datetime.tzinfo | None = UTC
    ) -> Self:
        tz = t.tzinfo or tz

        if tz is not None:
            tz = pendulum._safe_timezone(tz)

        return cls(t.hour, t.minute, t.second, t.microsecond, tzinfo=tz, fold=t.fold)

    # String formatting
    def __repr__(self) -> str:
        us = ""
        if self.microsecond:
            us = f", {self.microsecond}"

        tzinfo = ""
        if self.tzinfo:
            tzinfo = f", tzinfo={self.tzinfo!r}"

        return (
            f"{self.__class__.__name__}"
            f"({self.hour}, {self.minute}, {self.second}{us}{tzinfo})"
        )

    # Comparisons

    def closest(self, dt1: Time | time, dt2: Time | time) -> Self:
        """
        Get the closest time from the instance.
        """
        dt1 = self.__class__(dt1.hour, dt1.minute, dt1.second, dt1.microsecond)
        dt2 = self.__class__(dt2.hour, dt2.minute, dt2.second, dt2.microsecond)

        if self.diff(dt1).in_seconds() < self.diff(dt2).in_seconds():
            return dt1

        return dt2

    def farthest(self, dt1: Time | time, dt2: Time | time) -> Self:
        """
        Get the farthest time from the instance.
        """
        dt1 = self.__class__(dt1.hour, dt1.minute, dt1.second, dt1.microsecond)
        dt2 = self.__class__(dt2.hour, dt2.minute, dt2.second, dt2.microsecond)

        if self.diff(dt1).in_seconds() > self.diff(dt2).in_seconds():
            return dt1

        return dt2

    # ADDITIONS AND SUBSTRACTIONS

    def add(
        self, hours: int = 0, minutes: int = 0, seconds: int = 0, microseconds: int = 0
    ) -> Time:
        """
        Add duration to the instance.

        :param hours: The number of hours
        :param minutes: The number of minutes
        :param seconds: The number of seconds
        :param microseconds: The number of microseconds
        """
        from pendulum.datetime import DateTime

        return (
            DateTime.EPOCH.at(self.hour, self.minute, self.second, self.microsecond)
            .add(
                hours=hours, minutes=minutes, seconds=seconds, microseconds=microseconds
            )
            .time()
        )

    def subtract(
        self, hours: int = 0, minutes: int = 0, seconds: int = 0, microseconds: int = 0
    ) -> Time:
        """
        Add duration to the instance.

        :param hours: The number of hours
        :type hours: int

        :param minutes: The number of minutes
        :type minutes: int

        :param seconds: The number of seconds
        :type seconds: int

        :param microseconds: The number of microseconds
        :type microseconds: int

        :rtype: Time
        """
        from pendulum.datetime import DateTime

        return (
            DateTime.EPOCH.at(self.hour, self.minute, self.second, self.microsecond)
            .subtract(
                hours=hours, minutes=minutes, seconds=seconds, microseconds=microseconds
            )
            .time()
        )

    def add_timedelta(self, delta: datetime.timedelta) -> Time:
        """
        Add timedelta duration to the instance.

        :param delta: The timedelta instance
        """
        if delta.days:
            raise TypeError("Cannot add timedelta with days to Time.")

        return self.add(seconds=delta.seconds, microseconds=delta.microseconds)

    def subtract_timedelta(self, delta: datetime.timedelta) -> Time:
        """
        Remove timedelta duration from the instance.

        :param delta: The timedelta instance
        """
        if delta.days:
            raise TypeError("Cannot subtract timedelta with days to Time.")

        return self.subtract(seconds=delta.seconds, microseconds=delta.microseconds)

    def __add__(self, other: datetime.timedelta) -> Time:
        if not isinstance(other, timedelta):
            return NotImplemented

        return self.add_timedelta(other)

    @overload
    def __sub__(self, other: time) -> pendulum.Duration: ...

    @overload
    def __sub__(self, other: datetime.timedelta) -> Time: ...

    def __sub__(self, other: time | datetime.timedelta) -> pendulum.Duration | Time:
        if not isinstance(other, (Time, time, timedelta)):
            return NotImplemented

        if isinstance(other, timedelta):
            return self.subtract_timedelta(other)

        if isinstance(other, time):
            if other.tzinfo is not None:
                raise TypeError("Cannot subtract aware times to or from Time.")

            other = self.__class__(
                other.hour, other.minute, other.second, other.microsecond
            )

        return other.diff(self, False)

    @overload
    def __rsub__(self, other: time) -> pendulum.Duration: ...

    @overload
    def __rsub__(self, other: datetime.timedelta) -> Time: ...

    def __rsub__(self, other: time | datetime.timedelta) -> pendulum.Duration | Time:
        if not isinstance(other, (Time, time)):
            return NotImplemented

        if isinstance(other, time):
            if other.tzinfo is not None:
                raise TypeError("Cannot subtract aware times to or from Time.")

            other = self.__class__(
                other.hour, other.minute, other.second, other.microsecond
            )

        return other.__sub__(self)

    # DIFFERENCES

    def diff(self, dt: time | None = None, abs: bool = True) -> Duration:
        """
        Returns the difference between two Time objects as an Duration.

        :param dt: The time to subtract from
        :param abs: Whether to return an absolute duration or not
        """
        if dt is None:
            dt = pendulum.now().time()
        else:
            dt = self.__class__(dt.hour, dt.minute, dt.second, dt.microsecond)

        us1 = (
            self.hour * SECS_PER_HOUR + self.minute * SECS_PER_MIN + self.second
        ) * USECS_PER_SEC

        us2 = (
            dt.hour * SECS_PER_HOUR + dt.minute * SECS_PER_MIN + dt.second
        ) * USECS_PER_SEC

        klass = Duration
        if abs:
            klass = AbsoluteDuration

        return klass(microseconds=us2 - us1)

    def diff_for_humans(
        self,
        other: time | None = None,
        absolute: bool = False,
        locale: str | None = None,
    ) -> str:
        """
        Get the difference in a human readable format in the current locale.

        :param dt: The time to subtract from
        :param absolute: removes time difference modifiers ago, after, etc
        :param locale: The locale to use for localization
        """
        is_now = other is None

        if is_now:
            other = pendulum.now().time()

        diff = self.diff(other)

        return pendulum.format_diff(diff, is_now, absolute, locale)

    # Compatibility methods

    def replace(
        self,
        hour: SupportsIndex | None = None,
        minute: SupportsIndex | None = None,
        second: SupportsIndex | None = None,
        microsecond: SupportsIndex | None = None,
        tzinfo: bool | datetime.tzinfo | Literal[True] | None = True,
        fold: int = 0,
    ) -> Self:
        if tzinfo is True:
            tzinfo = self.tzinfo

        hour = hour if hour is not None else self.hour
        minute = minute if minute is not None else self.minute
        second = second if second is not None else self.second
        microsecond = microsecond if microsecond is not None else self.microsecond

        t = super().replace(
            hour,
            minute,
            second,
            microsecond,
            tzinfo=cast("Optional[datetime.tzinfo]", tzinfo),
            fold=fold,
        )
        return self.__class__(
            t.hour, t.minute, t.second, t.microsecond, tzinfo=t.tzinfo
        )

    def __getnewargs__(self) -> tuple[Time]:
        return (self,)

    def _get_state(
        self, protocol: SupportsIndex = 3
    ) -> tuple[int, int, int, int, datetime.tzinfo | None]:
        tz = self.tzinfo

        return self.hour, self.minute, self.second, self.microsecond, tz

    def __reduce__(
        self,
    ) -> tuple[type[Time], tuple[int, int, int, int, datetime.tzinfo | None]]:
        return self.__reduce_ex__(2)

    def __reduce_ex__(
        self, protocol: SupportsIndex
    ) -> tuple[type[Time], tuple[int, int, int, int, datetime.tzinfo | None]]:
        return self.__class__, self._get_state(protocol)


Time.min = Time(0, 0, 0)
Time.max = Time(23, 59, 59, 999999)
Time.resolution = Duration(microseconds=1)



from __future__ import annotations

import pendulum


locale = "nb"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "for 1 sekund siden"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "for 2 sekunder siden"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "for 1 minutt siden"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "for 2 minutter siden"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "for 1 time siden"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "for 2 timer siden"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "for 1 dag siden"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "for 2 dager siden"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "for 1 uke siden"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "for 2 uker siden"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "for 1 måned siden"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "for 2 måneder siden"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "for 1 år siden"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "for 2 år siden"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "om 1 sekund"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "1 sekund etter"
    assert d2.diff_for_humans(d, locale=locale) == "1 sekund før"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 sekund"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 sekunder"


def test_format():
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123456)
    assert d.format("dddd", locale=locale) == "søndag"
    assert d.format("ddd", locale=locale) == "søn."
    assert d.format("MMMM", locale=locale) == "august"
    assert d.format("MMM", locale=locale) == "aug."
    assert d.format("A", locale=locale) == "a.m."
    assert d.format("Qo", locale=locale) == "3."
    assert d.format("Mo", locale=locale) == "8."
    assert d.format("Do", locale=locale) == "28."

    assert d.format("LT", locale=locale) == "07:03"
    assert d.format("LTS", locale=locale) == "07:03:06"
    assert d.format("L", locale=locale) == "28.08.2016"
    assert d.format("LL", locale=locale) == "28. august 2016"
    assert d.format("LLL", locale=locale) == "28. august 2016 07:03"
    assert d.format("LLLL", locale=locale) == "søndag 28. august 2016 07:03"



from __future__ import annotations

import pendulum


locale = "nn"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "for 1 sekund sidan"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "for 2 sekund sidan"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "for 1 minutt sidan"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "for 2 minutt sidan"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "for 1 time sidan"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "for 2 timar sidan"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "for 1 dag sidan"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "for 2 dagar sidan"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "for 1 veke sidan"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "for 2 veker sidan"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "for 1 månad sidan"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "for 2 månadar sidan"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "for 1 år sidan"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "for 2 år sidan"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "om 1 sekund"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "1 sekund etter"
    assert d2.diff_for_humans(d, locale=locale) == "1 sekund før"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 sekund"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 sekund"


def test_format():
    d = pendulum.datetime(2016, 8, 29, 7, 3, 6, 123456)
    assert d.format("dddd", locale=locale) == "måndag"
    assert d.format("ddd", locale=locale) == "mån."
    assert d.format("MMMM", locale=locale) == "august"
    assert d.format("MMM", locale=locale) == "aug."
    assert d.format("A", locale=locale) == "formiddag"
    assert d.format("Qo", locale=locale) == "3."
    assert d.format("Mo", locale=locale) == "8."
    assert d.format("Do", locale=locale) == "29."

    assert d.format("LT", locale=locale) == "07:03"
    assert d.format("LTS", locale=locale) == "07:03:06"
    assert d.format("L", locale=locale) == "29.08.2016"
    assert d.format("LL", locale=locale) == "29. august 2016"
    assert d.format("LLL", locale=locale) == "29. august 2016 07:03"
    assert d.format("LLLL", locale=locale) == "måndag 29. august 2016 07:03"



from __future__ import annotations

import pytest

import pendulum

from pendulum.exceptions import PendulumException
from tests.conftest import assert_datetime


def test_start_of_week():
    d = pendulum.datetime(1980, 8, 7, 12, 11, 9).start_of("week")
    assert_datetime(d, 1980, 8, 4, 0, 0, 0)


def test_start_of_week_from_week_start():
    d = pendulum.datetime(1980, 8, 4).start_of("week")
    assert_datetime(d, 1980, 8, 4, 0, 0, 0)


def test_start_of_week_crossing_year_boundary():
    d = pendulum.datetime(2014, 1, 1).start_of("week")
    assert_datetime(d, 2013, 12, 30, 0, 0, 0)


def test_end_of_week():
    d = pendulum.datetime(1980, 8, 7, 12, 11, 9).end_of("week")
    assert_datetime(d, 1980, 8, 10, 23, 59, 59)


def test_end_of_week_from_week_end():
    d = pendulum.datetime(1980, 8, 10).end_of("week")
    assert_datetime(d, 1980, 8, 10, 23, 59, 59)


def test_end_of_week_crossing_year_boundary():
    d = pendulum.datetime(2013, 12, 31).end_of("week")
    assert_datetime(d, 2014, 1, 5, 23, 59, 59)


def test_next():
    d = pendulum.datetime(1975, 5, 21).next()
    assert_datetime(d, 1975, 5, 28, 0, 0, 0)


def test_next_monday():
    d = pendulum.datetime(1975, 5, 21).next(pendulum.MONDAY)
    assert_datetime(d, 1975, 5, 26, 0, 0, 0)


def test_next_saturday():
    d = pendulum.datetime(1975, 5, 21).next(5)
    assert_datetime(d, 1975, 5, 24, 0, 0, 0)


def test_next_keep_time():
    d = pendulum.datetime(1975, 5, 21, 12).next()
    assert_datetime(d, 1975, 5, 28, 0, 0, 0)

    d = pendulum.datetime(1975, 5, 21, 12).next(keep_time=True)
    assert_datetime(d, 1975, 5, 28, 12, 0, 0)


def test_next_invalid():
    dt = pendulum.datetime(1975, 5, 21, 12)

    with pytest.raises(ValueError):
        dt.next(7)


def test_previous():
    d = pendulum.datetime(1975, 5, 21).previous()
    assert_datetime(d, 1975, 5, 14, 0, 0, 0)


def test_previous_monday():
    d = pendulum.datetime(1975, 5, 21).previous(pendulum.MONDAY)
    assert_datetime(d, 1975, 5, 19, 0, 0, 0)


def test_previous_saturday():
    d = pendulum.datetime(1975, 5, 21).previous(5)
    assert_datetime(d, 1975, 5, 17, 0, 0, 0)


def test_previous_keep_time():
    d = pendulum.datetime(1975, 5, 21, 12).previous()
    assert_datetime(d, 1975, 5, 14, 0, 0, 0)

    d = pendulum.datetime(1975, 5, 21, 12).previous(keep_time=True)
    assert_datetime(d, 1975, 5, 14, 12, 0, 0)


def test_previous_invalid():
    dt = pendulum.datetime(1975, 5, 21, 12)

    with pytest.raises(ValueError):
        dt.previous(7)


def test_first_day_of_month():
    d = pendulum.datetime(1975, 11, 21).first_of("month")
    assert_datetime(d, 1975, 11, 1, 0, 0, 0)


def test_first_wednesday_of_month():
    d = pendulum.datetime(1975, 11, 21).first_of("month", pendulum.WEDNESDAY)
    assert_datetime(d, 1975, 11, 5, 0, 0, 0)


def test_first_friday_of_month():
    d = pendulum.datetime(1975, 11, 21).first_of("month", 4)
    assert_datetime(d, 1975, 11, 7, 0, 0, 0)


def test_last_day_of_month():
    d = pendulum.datetime(1975, 12, 5).last_of("month")
    assert_datetime(d, 1975, 12, 31, 0, 0, 0)


def test_last_tuesday_of_month():
    d = pendulum.datetime(1975, 12, 1).last_of("month", pendulum.TUESDAY)
    assert_datetime(d, 1975, 12, 30, 0, 0, 0)


def test_last_friday_of_month():
    d = pendulum.datetime(1975, 12, 5).last_of("month", 4)
    assert_datetime(d, 1975, 12, 26, 0, 0, 0)


def test_nth_of_month_outside_scope():
    d = pendulum.datetime(1975, 6, 5)

    with pytest.raises(PendulumException):
        d.nth_of("month", 6, pendulum.MONDAY)


def test_nth_of_month_outside_year():
    d = pendulum.datetime(1975, 12, 5)

    with pytest.raises(PendulumException):
        d.nth_of("month", 55, pendulum.MONDAY)


def test_nth_of_month_first():
    d = pendulum.datetime(1975, 12, 5).nth_of("month", 1, pendulum.MONDAY)

    assert_datetime(d, 1975, 12, 1, 0, 0, 0)


def test_2nd_monday_of_month():
    d = pendulum.datetime(1975, 12, 5).nth_of("month", 2, pendulum.MONDAY)

    assert_datetime(d, 1975, 12, 8, 0, 0, 0)


def test_3rd_wednesday_of_month():
    d = pendulum.datetime(1975, 12, 5).nth_of("month", 3, 2)

    assert_datetime(d, 1975, 12, 17, 0, 0, 0)


def test_first_day_of_quarter():
    d = pendulum.datetime(1975, 11, 21).first_of("quarter")
    assert_datetime(d, 1975, 10, 1, 0, 0, 0)


def test_first_wednesday_of_quarter():
    d = pendulum.datetime(1975, 11, 21).first_of("quarter", pendulum.WEDNESDAY)
    assert_datetime(d, 1975, 10, 1, 0, 0, 0)


def test_first_friday_of_quarter():
    d = pendulum.datetime(1975, 11, 21).first_of("quarter", 4)
    assert_datetime(d, 1975, 10, 3, 0, 0, 0)


def test_first_of_quarter_from_a_day_that_will_not_exist_in_the_first_month():
    d = pendulum.datetime(2014, 5, 31).first_of("quarter")
    assert_datetime(d, 2014, 4, 1, 0, 0, 0)


def test_last_day_of_quarter():
    d = pendulum.datetime(1975, 8, 5).last_of("quarter")
    assert_datetime(d, 1975, 9, 30, 0, 0, 0)


def test_last_tuesday_of_quarter():
    d = pendulum.datetime(1975, 8, 5).last_of("quarter", pendulum.TUESDAY)
    assert_datetime(d, 1975, 9, 30, 0, 0, 0)


def test_last_friday_of_quarter():
    d = pendulum.datetime(1975, 8, 5).last_of("quarter", pendulum.FRIDAY)
    assert_datetime(d, 1975, 9, 26, 0, 0, 0)


def test_last_day_of_quarter_that_will_not_exist_in_the_last_month():
    d = pendulum.datetime(2014, 5, 31).last_of("quarter")
    assert_datetime(d, 2014, 6, 30, 0, 0, 0)


def test_nth_of_quarter_outside_scope():
    d = pendulum.datetime(1975, 1, 5)

    with pytest.raises(PendulumException):
        d.nth_of("quarter", 20, pendulum.MONDAY)


def test_nth_of_quarter_outside_year():
    d = pendulum.datetime(1975, 1, 5)

    with pytest.raises(PendulumException):
        d.nth_of("quarter", 55, pendulum.MONDAY)


def test_nth_of_quarter_first():
    d = pendulum.datetime(1975, 12, 5).nth_of("quarter", 1, pendulum.MONDAY)

    assert_datetime(d, 1975, 10, 6, 0, 0, 0)


def test_nth_of_quarter_from_a_day_that_will_not_exist_in_the_first_month():
    d = pendulum.datetime(2014, 5, 31).nth_of("quarter", 2, pendulum.MONDAY)
    assert_datetime(d, 2014, 4, 14, 0, 0, 0)


def test_2nd_monday_of_quarter():
    d = pendulum.datetime(1975, 8, 5).nth_of("quarter", 2, pendulum.MONDAY)
    assert_datetime(d, 1975, 7, 14, 0, 0, 0)


def test_3rd_wednesday_of_quarter():
    d = pendulum.datetime(1975, 8, 5).nth_of("quarter", 3, 2)
    assert_datetime(d, 1975, 7, 16, 0, 0, 0)


def test_first_day_of_year():
    d = pendulum.datetime(1975, 11, 21).first_of("year")
    assert_datetime(d, 1975, 1, 1, 0, 0, 0)


def test_first_wednesday_of_year():
    d = pendulum.datetime(1975, 11, 21).first_of("year", pendulum.WEDNESDAY)
    assert_datetime(d, 1975, 1, 1, 0, 0, 0)


def test_first_friday_of_year():
    d = pendulum.datetime(1975, 11, 21).first_of("year", 4)
    assert_datetime(d, 1975, 1, 3, 0, 0, 0)


def test_last_day_of_year():
    d = pendulum.datetime(1975, 8, 5).last_of("year")
    assert_datetime(d, 1975, 12, 31, 0, 0, 0)


def test_last_tuesday_of_year():
    d = pendulum.datetime(1975, 8, 5).last_of("year", pendulum.TUESDAY)
    assert_datetime(d, 1975, 12, 30, 0, 0, 0)


def test_last_friday_of_year():
    d = pendulum.datetime(1975, 8, 5).last_of("year", 4)
    assert_datetime(d, 1975, 12, 26, 0, 0, 0)


def test_nth_of_year_outside_scope():
    d = pendulum.datetime(1975, 1, 5)

    with pytest.raises(PendulumException):
        d.nth_of("year", 55, pendulum.MONDAY)


def test_nth_of_year_first():
    d = pendulum.datetime(1975, 12, 5).nth_of("year", 1, pendulum.MONDAY)

    assert_datetime(d, 1975, 1, 6, 0, 0, 0)


def test_2nd_monday_of_year():
    d = pendulum.datetime(1975, 8, 5).nth_of("year", 2, pendulum.MONDAY)
    assert_datetime(d, 1975, 1, 13, 0, 0, 0)


def test_2rd_wednesday_of_year():
    d = pendulum.datetime(1975, 8, 5).nth_of("year", 3, pendulum.WEDNESDAY)
    assert_datetime(d, 1975, 1, 15, 0, 0, 0)


def test_7th_thursday_of_year():
    d = pendulum.datetime(1975, 8, 31).nth_of("year", 7, pendulum.THURSDAY)
    assert_datetime(d, 1975, 2, 13, 0, 0, 0)


def test_first_of_invalid_unit():
    d = pendulum.datetime(1975, 8, 5)

    with pytest.raises(ValueError):
        d.first_of("invalid")


def test_last_of_invalid_unit():
    d = pendulum.datetime(1975, 8, 5)

    with pytest.raises(ValueError):
        d.last_of("invalid")


def test_nth_of_invalid_unit():
    d = pendulum.datetime(1975, 8, 5)

    with pytest.raises(ValueError):
        d.nth_of("invalid", 3, pendulum.MONDAY)



from __future__ import annotations

import pendulum


locale = "lt"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "prieš 1 sekundę"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "prieš 2 sekundes"

    d = pendulum.now().subtract(seconds=21)
    assert d.diff_for_humans(locale=locale) == "prieš 21 sekundę"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "prieš 1 minutę"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "prieš 2 minutes"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "prieš 1 valandą"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "prieš 2 valandas"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "prieš 1 dieną"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "prieš 2 dienas"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "prieš 1 savaitę"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "prieš 2 savaites"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "prieš 1 mėnesį"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "prieš 2 mėnesius"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "prieš 1 metus"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "prieš 2 metus"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "po 1 sekundės"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "po 1 sekundės"
    assert d2.diff_for_humans(d, locale=locale) == "1 sekundę nuo dabar"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 sekundė"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 sekundės"



from __future__ import annotations

import pendulum


locale = "it"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "alcuni secondi fa"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "alcuni secondi fa"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "1 minuto fa"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "2 minuti fa"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "1 ora fa"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "2 ore fa"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "1 giorno fa"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "2 giorni fa"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "1 settimana fa"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "2 settimane fa"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "1 mese fa"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "2 mesi fa"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "1 anno fa"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "2 anni fa"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "in alcuni secondi"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "alcuni secondi dopo"
    assert d2.diff_for_humans(d, locale=locale) == "alcuni secondi prima"

    assert d.diff_for_humans(d2, True, locale=locale) == "alcuni secondi"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "alcuni secondi"


def test_format():
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123456)
    assert d.format("dddd", locale=locale) == "domenica"
    assert d.format("ddd", locale=locale) == "dom"
    assert d.format("MMMM", locale=locale) == "agosto"
    assert d.format("MMM", locale=locale) == "ago"
    assert d.format("A", locale=locale) == "AM"

    assert d.format("LT", locale=locale) == "7:03"
    assert d.format("LTS", locale=locale) == "7:03:06"
    assert d.format("L", locale=locale) == "28/08/2016"
    assert d.format("LL", locale=locale) == "28 agosto 2016"
    assert d.format("LLL", locale=locale) == "28 agosto 2016 alle 7:03"
    assert d.format("LLLL", locale=locale) == "domenica, 28 agosto 2016 alle 7:03"

    assert d.format("Do", locale=locale) == "28°"
    d = pendulum.datetime(2019, 1, 1, 7, 3, 6, 123456)
    assert d.format("Do", locale=locale) == "1°"



from __future__ import annotations

from pendulum import Time
from tests.conftest import assert_time


def test_replace():
    t = Time(12, 34, 56, 123456)
    t = t.replace(1, 2, 3, 654321)

    assert isinstance(t, Time)
    assert_time(t, 1, 2, 3, 654321)



from __future__ import annotations

from pendulum.formatting import Formatter


_formatter = Formatter()


class FormattableMixin:
    _formatter: Formatter = _formatter

    def format(self, fmt: str, locale: str | None = None) -> str:
        """
        Formats the instance using the given format.

        :param fmt: The format to use
        :param locale: The locale to use
        """
        return self._formatter.format(self, fmt, locale)

    def for_json(self) -> str:
        """
        Methods for automatic json serialization by simplejson.
        """
        return self.isoformat()

    def __format__(self, format_spec: str) -> str:
        if len(format_spec) > 0:
            if "%" in format_spec:
                return self.strftime(format_spec)

            return self.format(format_spec)

        return str(self)

    def __str__(self) -> str:
        return self.isoformat()



from __future__ import annotations

import datetime

import pytest

import pendulum

from pendulum.parsing import ParserError
from pendulum.parsing import parse


def test_y():
    text = "2016"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 1
    assert parsed.day == 1
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None


def test_ym():
    text = "2016-10"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 1
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None


def test_ymd():
    text = "2016-10-06"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 6
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None


def test_ymd_one_character():
    text = "2016-2-6"

    parsed = parse(text, strict=False)

    assert parsed.year == 2016
    assert parsed.month == 2
    assert parsed.day == 6
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None


def test_ymd_hms():
    text = "2016-10-06 12:34:56"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 6
    assert parsed.hour == 12
    assert parsed.minute == 34
    assert parsed.second == 56
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2016-10-06 12:34:56.123456"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 6
    assert parsed.hour == 12
    assert parsed.minute == 34
    assert parsed.second == 56
    assert parsed.microsecond == 123456
    assert parsed.tzinfo is None


def test_rfc_3339():
    text = "2016-10-06T12:34:56+05:30"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 6
    assert parsed.hour == 12
    assert parsed.minute == 34
    assert parsed.second == 56
    assert parsed.microsecond == 0
    assert parsed.utcoffset().total_seconds() == 19800


def test_rfc_3339_extended():
    text = "2016-10-06T12:34:56.123456+05:30"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 6
    assert parsed.hour == 12
    assert parsed.minute == 34
    assert parsed.second == 56
    assert parsed.microsecond == 123456
    assert parsed.utcoffset().total_seconds() == 19800

    text = "2016-10-06T12:34:56.000123+05:30"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 6
    assert parsed.hour == 12
    assert parsed.minute == 34
    assert parsed.second == 56
    assert parsed.microsecond == 123
    assert parsed.utcoffset().total_seconds() == 19800


def test_rfc_3339_extended_nanoseconds():
    text = "2016-10-06T12:34:56.123456789+05:30"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 6
    assert parsed.hour == 12
    assert parsed.minute == 34
    assert parsed.second == 56
    assert parsed.microsecond == 123456
    assert parsed.utcoffset().total_seconds() == 19800


def test_iso_8601_date():
    text = "2012"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 1
    assert parsed.day == 1
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2012-05-03"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 5
    assert parsed.day == 3
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "20120503"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 5
    assert parsed.day == 3
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2012-05"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 5
    assert parsed.day == 1
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None


def test_iso8601_datetime():
    text = "2016-10-01T14"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 1
    assert parsed.hour == 14
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2016-10-01T14:30"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 1
    assert parsed.hour == 14
    assert parsed.minute == 30
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "20161001T14"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 1
    assert parsed.hour == 14
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "20161001T1430"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 1
    assert parsed.hour == 14
    assert parsed.minute == 30
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "20161001T1430+0530"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 10
    assert parsed.day == 1
    assert parsed.hour == 14
    assert parsed.minute == 30
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.utcoffset().total_seconds() == 19800

    text = "2008-09-03T20:56:35.450686+01"

    parsed = parse(text)

    assert parsed.year == 2008
    assert parsed.month == 9
    assert parsed.day == 3
    assert parsed.hour == 20
    assert parsed.minute == 56
    assert parsed.second == 35
    assert parsed.microsecond == 450686
    assert parsed.utcoffset().total_seconds() == 3600


def test_iso8601_week_number():
    text = "2012-W05"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 1
    assert parsed.day == 30
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2012W05"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 1
    assert parsed.day == 30
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    # Long Year
    text = "2015W53"

    parsed = parse(text)

    assert parsed.year == 2015
    assert parsed.month == 12
    assert parsed.day == 28
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2012-W05-5"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 2
    assert parsed.day == 3
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2012W055"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 2
    assert parsed.day == 3
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2009-W53-7"
    parsed = parse(text)

    assert parsed.year == 2010
    assert parsed.month == 1
    assert parsed.day == 3
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2009-W01-1"
    parsed = parse(text)

    assert parsed.year == 2008
    assert parsed.month == 12
    assert parsed.day == 29
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None


def test_iso8601_week_number_with_time():
    text = "2012-W05T09"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 1
    assert parsed.day == 30
    assert parsed.hour == 9
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2012W05T09"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 1
    assert parsed.day == 30
    assert parsed.hour == 9
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2012-W05-5T09"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 2
    assert parsed.day == 3
    assert parsed.hour == 9
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2012W055T09"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 2
    assert parsed.day == 3
    assert parsed.hour == 9
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None


def test_iso8601_ordinal():
    text = "2012-007"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 1
    assert parsed.day == 7
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "2012007"

    parsed = parse(text)

    assert parsed.year == 2012
    assert parsed.month == 1
    assert parsed.day == 7
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None


def test_iso8601_time():
    now = pendulum.datetime(2015, 11, 12)

    text = "T201205"

    parsed = parse(text, now=now)

    assert parsed.year == 2015
    assert parsed.month == 11
    assert parsed.day == 12
    assert parsed.hour == 20
    assert parsed.minute == 12
    assert parsed.second == 5
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "20:12:05"

    parsed = parse(text, now=now)

    assert parsed.year == 2015
    assert parsed.month == 11
    assert parsed.day == 12
    assert parsed.hour == 20
    assert parsed.minute == 12
    assert parsed.second == 5
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "20:12:05.123456"

    parsed = parse(text, now=now)

    assert parsed.year == 2015
    assert parsed.month == 11
    assert parsed.day == 12
    assert parsed.hour == 20
    assert parsed.minute == 12
    assert parsed.second == 5
    assert parsed.microsecond == 123456
    assert parsed.tzinfo is None


def test_iso8601_ordinal_invalid():
    text = "2012-007-05"

    with pytest.raises(ParserError):
        parse(text)


def test_exact():
    text = "2012"

    parsed = parse(text, exact=True)

    assert isinstance(parsed, datetime.date)
    assert parsed.year == 2012
    assert parsed.month == 1
    assert parsed.day == 1

    text = "2012-03"

    parsed = parse(text, exact=True)

    assert isinstance(parsed, datetime.date)
    assert parsed.year == 2012
    assert parsed.month == 3
    assert parsed.day == 1

    text = "2012-03-13"

    parsed = parse(text, exact=True)

    assert isinstance(parsed, datetime.date)
    assert parsed.year == 2012
    assert parsed.month == 3
    assert parsed.day == 13

    text = "2012W055"

    parsed = parse(text, exact=True)

    assert isinstance(parsed, datetime.date)
    assert parsed.year == 2012
    assert parsed.month == 2
    assert parsed.day == 3

    text = "2012007"

    parsed = parse(text, exact=True)

    assert isinstance(parsed, datetime.date)
    assert parsed.year == 2012
    assert parsed.month == 1
    assert parsed.day == 7

    text = "20:12:05"

    parsed = parse(text, exact=True)

    assert isinstance(parsed, datetime.time)
    assert parsed.hour == 20
    assert parsed.minute == 12
    assert parsed.second == 5
    assert parsed.microsecond == 0


def test_edge_cases():
    text = "2013-11-1"

    parsed = parse(text, strict=False)
    assert parsed.year == 2013
    assert parsed.month == 11
    assert parsed.day == 1
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "10-01-01"

    parsed = parse(text, strict=False)
    assert parsed.year == 2010
    assert parsed.month == 1
    assert parsed.day == 1
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "31-01-01"

    parsed = parse(text, strict=False)
    assert parsed.year == 2031
    assert parsed.month == 1
    assert parsed.day == 1
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None

    text = "32-01-01"

    parsed = parse(text, strict=False)
    assert parsed.year == 2032
    assert parsed.month == 1
    assert parsed.day == 1
    assert parsed.hour == 0
    assert parsed.minute == 0
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None


def test_strict():
    text = "4 Aug 2015 - 11:20 PM"

    with pytest.raises(ParserError):
        parse(text)

    parsed = parse(text, strict=False)
    assert parsed.year == 2015
    assert parsed.month == 8
    assert parsed.day == 4
    assert parsed.hour == 23
    assert parsed.minute == 20
    assert parsed.second == 0
    assert parsed.microsecond == 0
    assert parsed.tzinfo is None


def test_invalid():
    text = "201610T"

    with pytest.raises(ParserError):
        parse(text)

    text = "2012-W54"

    with pytest.raises(ParserError):
        parse(text)

    text = "2012-W13-8"

    with pytest.raises(ParserError):
        parse(text)

    # W53 in normal year (not long)
    text = "2017W53"

    with pytest.raises(ParserError):
        parse(text)

    text = "/2012"

    with pytest.raises(ParserError):
        parse(text)

    text = "2012/"

    with pytest.raises(ParserError):
        parse(text)


def test_exif_edge_case():
    text = "2016:12:26 15:45:28"

    parsed = parse(text)

    assert parsed.year == 2016
    assert parsed.month == 12
    assert parsed.day == 26
    assert parsed.hour == 15
    assert parsed.minute == 45
    assert parsed.second == 28



"""
sv custom locale file.
"""
from __future__ import annotations


translations = {
    # Relative time
    "ago": "{} sedan",
    "from_now": "från nu {}",
    "after": "{0} efter",
    "before": "{0} innan",
    # Date formats
    "date_formats": {
        "LTS": "HH:mm:ss",
        "LT": "HH:mm",
        "L": "YYYY-MM-DD",
        "LL": "D MMMM YYYY",
        "LLL": "D MMMM YYYY, HH:mm",
        "LLLL": "dddd, D MMMM YYYY, HH:mm",
    },
}



# mypy: no-warn-redundant-casts
from __future__ import annotations

import datetime as _datetime
import zoneinfo

from abc import ABC
from abc import abstractmethod
from typing import TYPE_CHECKING
from typing import TypeVar
from typing import cast

from pendulum.tz.exceptions import AmbiguousTime
from pendulum.tz.exceptions import InvalidTimezone
from pendulum.tz.exceptions import NonExistingTime


if TYPE_CHECKING:
    from typing_extensions import Self

POST_TRANSITION = "post"
PRE_TRANSITION = "pre"
TRANSITION_ERROR = "error"


_DT = TypeVar("_DT", bound=_datetime.datetime)


class PendulumTimezone(ABC):
    @property
    @abstractmethod
    def name(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def convert(self, dt: _DT, raise_on_unknown_times: bool = False) -> _DT:
        raise NotImplementedError

    @abstractmethod
    def datetime(
        self,
        year: int,
        month: int,
        day: int,
        hour: int = 0,
        minute: int = 0,
        second: int = 0,
        microsecond: int = 0,
    ) -> _datetime.datetime:
        raise NotImplementedError


class Timezone(zoneinfo.ZoneInfo, PendulumTimezone):
    """
    Represents a named timezone.

    The accepted names are those provided by the IANA time zone database.

    >>> from pendulum.tz.timezone import Timezone
    >>> tz = Timezone('Europe/Paris')
    """

    def __new__(cls, key: str) -> Self:
        try:
            return super().__new__(cls, key)  # type: ignore[call-arg]
        except zoneinfo.ZoneInfoNotFoundError:
            raise InvalidTimezone(key)

    def __eq__(self, other: object) -> bool:
        return isinstance(other, Timezone) and self.key == other.key

    @property
    def name(self) -> str:
        return self.key

    def convert(self, dt: _DT, raise_on_unknown_times: bool = False) -> _DT:
        """
        Converts a datetime in the current timezone.

        If the datetime is naive, it will be normalized.

        >>> from datetime import datetime
        >>> from pendulum import timezone
        >>> paris = timezone('Europe/Paris')
        >>> dt = datetime(2013, 3, 31, 2, 30, fold=1)
        >>> in_paris = paris.convert(dt)
        >>> in_paris.isoformat()
        '2013-03-31T03:30:00+02:00'

        If the datetime is aware, it will be properly converted.

        >>> new_york = timezone('America/New_York')
        >>> in_new_york = new_york.convert(in_paris)
        >>> in_new_york.isoformat()
        '2013-03-30T21:30:00-04:00'
        """

        if dt.tzinfo is None:
            # Technically, utcoffset() can return None, but none of the zone information
            # in tzdata sets _tti_before to None. This can be checked with the following
            # code:
            #
            # >>> import zoneinfo
            # >>> from zoneinfo._zoneinfo import ZoneInfo
            #
            # >>> for tzname in zoneinfo.available_timezones():
            # >>>     if ZoneInfo(tzname)._tti_before is None:
            # >>>         print(tzname)

            offset_before = cast(
                "_datetime.timedelta",
                (self.utcoffset(dt.replace(fold=0)) if dt.fold else self.utcoffset(dt)),
            )
            offset_after = cast(
                "_datetime.timedelta",
                (self.utcoffset(dt) if dt.fold else self.utcoffset(dt.replace(fold=1))),
            )

            if offset_after > offset_before:
                # Skipped time
                if raise_on_unknown_times:
                    raise NonExistingTime(dt)

                dt = cast(
                    "_DT",
                    dt
                    + (
                        (offset_after - offset_before)
                        if dt.fold
                        else (offset_before - offset_after)
                    ),
                )
            elif offset_before > offset_after and raise_on_unknown_times:
                # Repeated time
                raise AmbiguousTime(dt)

            return dt.replace(tzinfo=self)

        return cast("_DT", dt.astimezone(self))

    def datetime(
        self,
        year: int,
        month: int,
        day: int,
        hour: int = 0,
        minute: int = 0,
        second: int = 0,
        microsecond: int = 0,
    ) -> _datetime.datetime:
        """
        Return a normalized datetime for the current timezone.
        """
        return self.convert(
            _datetime.datetime(
                year, month, day, hour, minute, second, microsecond, fold=1
            )
        )

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}('{self.name}')"


class FixedTimezone(_datetime.tzinfo, PendulumTimezone):
    def __init__(self, offset: int, name: str | None = None) -> None:
        sign = "-" if offset < 0 else "+"

        minutes = offset / 60
        hour, minute = divmod(abs(int(minutes)), 60)

        if not name:
            name = f"{sign}{hour:02d}:{minute:02d}"

        self._name = name
        self._offset = offset
        self._utcoffset = _datetime.timedelta(seconds=offset)

    def __eq__(self, other: object) -> bool:
        return isinstance(other, FixedTimezone) and self._offset == other._offset

    @property
    def name(self) -> str:
        return self._name

    def convert(self, dt: _DT, raise_on_unknown_times: bool = False) -> _DT:
        if dt.tzinfo is None:
            return dt.__class__(
                dt.year,
                dt.month,
                dt.day,
                dt.hour,
                dt.minute,
                dt.second,
                dt.microsecond,
                tzinfo=self,
                fold=0,
            )

        return cast("_DT", dt.astimezone(self))

    def datetime(
        self,
        year: int,
        month: int,
        day: int,
        hour: int = 0,
        minute: int = 0,
        second: int = 0,
        microsecond: int = 0,
    ) -> _datetime.datetime:
        return self.convert(
            _datetime.datetime(
                year, month, day, hour, minute, second, microsecond, fold=1
            )
        )

    @property
    def offset(self) -> int:
        return self._offset

    def utcoffset(self, dt: _datetime.datetime | None) -> _datetime.timedelta:
        return self._utcoffset

    def dst(self, dt: _datetime.datetime | None) -> _datetime.timedelta:
        return _datetime.timedelta()

    def fromutc(self, dt: _datetime.datetime) -> _datetime.datetime:
        # Use the stdlib datetime's add method to avoid infinite recursion
        return (_datetime.datetime.__add__(dt, self._utcoffset)).replace(tzinfo=self)

    def tzname(self, dt: _datetime.datetime | None) -> str | None:
        return self._name

    def __getinitargs__(self) -> tuple[int, str]:
        return self._offset, self._name

    def __repr__(self) -> str:
        name = ""
        if self._name:
            name = f', name="{self._name}"'

        return f"{self.__class__.__name__}({self._offset}{name})"


UTC = Timezone("UTC")



from __future__ import annotations

import pendulum


locale = "sv"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "för 1 sekund sedan"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "för 2 sekunder sedan"

    d = pendulum.now().subtract(seconds=5)
    assert d.diff_for_humans(locale=locale) == "för 5 sekunder sedan"

    d = pendulum.now().subtract(seconds=21)
    assert d.diff_for_humans(locale=locale) == "för 21 sekunder sedan"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "för 1 minut sedan"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "för 2 minuter sedan"

    d = pendulum.now().subtract(minutes=5)
    assert d.diff_for_humans(locale=locale) == "för 5 minuter sedan"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "för 1 timme sedan"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "för 2 timmar sedan"

    d = pendulum.now().subtract(hours=5)
    assert d.diff_for_humans(locale=locale) == "för 5 timmar sedan"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "för 1 dag sedan"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "för 2 dagar sedan"

    d = pendulum.now().subtract(days=5)
    assert d.diff_for_humans(locale=locale) == "för 5 dagar sedan"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "för 1 vecka sedan"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "för 2 veckor sedan"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "för 1 månad sedan"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "för 2 månader sedan"

    d = pendulum.now().subtract(months=5)
    assert d.diff_for_humans(locale=locale) == "för 5 månader sedan"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "för 1 år sedan"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "för 2 år sedan"

    d = pendulum.now().subtract(years=5)
    assert d.diff_for_humans(locale=locale) == "för 5 år sedan"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "om 1 sekund"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "1 sekund efter"
    assert d2.diff_for_humans(d, locale=locale) == "1 sekund innan"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 sekund"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 sekunder"



from __future__ import annotations

import pytest

import pendulum


def test_timezones():
    zones = pendulum.timezones()

    assert "America/Argentina/Buenos_Aires" in zones


@pytest.mark.parametrize("zone", list(pendulum.timezones()))
def test_timezones_are_loadable(zone):
    pendulum.timezone(zone)



from __future__ import annotations

import pendulum


locale = "sk"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "pred 1 sekundou"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "o 1 sekundu"

    d = pendulum.now().add(seconds=2)
    assert d.diff_for_humans(locale=locale) == "o 2 sekundy"

    d = pendulum.now().add(seconds=5)
    assert d.diff_for_humans(locale=locale) == "o 5 sekúnd"

    d = pendulum.now().subtract(seconds=20)
    assert d.diff_for_humans(locale=locale) == "pred 20 sekundami"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "pred 1 minútou"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "pred 2 minútami"

    d = pendulum.now().add(minutes=5)
    assert d.diff_for_humans(locale=locale) == "o 5 minút"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "pred 1 hodinou"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "pred 2 hodinami"

    d = pendulum.now().subtract(hours=5)
    assert d.diff_for_humans(locale=locale) == "pred 5 hodinami"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "pred 1 dňom"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "pred 2 dňami"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "pred 1 týždňom"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "pred 2 týždňami"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "pred 1 mesiacom"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "pred 2 mesiacmi"

    d = pendulum.now().subtract(months=5)
    assert d.diff_for_humans(locale=locale) == "pred 5 mesiacmi"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "pred 1 rokom"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "pred 2 rokmi"

    d = pendulum.now().subtract(years=5)
    assert d.diff_for_humans(locale=locale) == "pred 5 rokmi"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "1 sekunda po"
    assert d2.diff_for_humans(d, locale=locale) == "1 sekunda pred"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 sekunda"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 sekundy"

    d = pendulum.now().add(seconds=20)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "20 sekúnd po"
    assert d2.diff_for_humans(d, locale=locale) == "20 sekúnd pred"

    d = pendulum.now().add(seconds=10)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, True, locale=locale) == "10 sekúnd"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "11 sekúnd"


def test_format():
    d = pendulum.datetime(2016, 8, 29, 7, 3, 6, 123456)
    assert d.format("dddd", locale=locale) == "pondelok"
    assert d.format("ddd", locale=locale) == "po"
    assert d.format("MMMM", locale=locale) == "augusta"
    assert d.format("MMM", locale=locale) == "aug"
    assert d.format("A", locale=locale) == "AM"
    assert d.format("Qo", locale=locale) == "3"
    assert d.format("Mo", locale=locale) == "8"
    assert d.format("Do", locale=locale) == "29"

    assert d.format("LT", locale=locale) == "07:03"
    assert d.format("LTS", locale=locale) == "07:03:06"
    assert d.format("L", locale=locale) == "29.08.2016"
    assert d.format("LL", locale=locale) == "29. augusta 2016"
    assert d.format("LLL", locale=locale) == "29. augusta 2016 07:03"
    assert d.format("LLLL", locale=locale) == "pondelok, 29. augusta 2016 07:03"



from __future__ import annotations

from datetime import date

import pytest

import pendulum


@pytest.fixture
def today():
    return pendulum.Date.today()


def test_diff_in_years_positive():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.add(years=1)).in_years() == 1


def test_diff_in_years_negative_with_sign():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.subtract(years=1), False).in_years() == -1


def test_diff_in_years_negative_no_sign():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.subtract(years=1)).in_years() == 1


def test_diff_in_years_vs_default_now(today):
    assert today.subtract(years=1).diff().in_years() == 1


def test_diff_in_years_ensure_is_truncated():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.add(years=1).add(months=7)).in_years() == 1


def test_diff_in_months_positive():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.add(years=1).add(months=1)).in_months() == 13


def test_diff_in_months_negative_with_sign():
    dt = pendulum.date(2000, 1, 1)

    assert dt.diff(dt.subtract(years=1).add(months=1), False).in_months() == -11


def test_diff_in_months_negative_no_sign():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.subtract(years=1).add(months=1)).in_months() == 11


def test_diff_in_months_vs_default_now(today):
    assert today.subtract(years=1).diff().in_months() == 12


def test_diff_in_months_ensure_is_truncated():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.add(months=1).add(days=16)).in_months() == 1


def test_diff_in_days_positive():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.add(years=1)).in_days() == 366


def test_diff_in_days_negative_with_sign():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.subtract(years=1), False).in_days() == -365


def test_diff_in_days_negative_no_sign():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.subtract(years=1)).in_days() == 365


def test_diff_in_days_vs_default_now(today):
    assert today.subtract(weeks=1).diff().in_days() == 7


def test_diff_in_weeks_positive():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.add(years=1)).in_weeks() == 52


def test_diff_in_weeks_negative_with_sign():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.subtract(years=1), False).in_weeks() == -52


def test_diff_in_weeks_negative_no_sign():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.subtract(years=1)).in_weeks() == 52


def test_diff_in_weeks_vs_default_now(today):
    assert today.subtract(weeks=1).diff().in_weeks() == 1


def test_diff_in_weeks_ensure_is_truncated():
    dt = pendulum.date(2000, 1, 1)
    assert dt.diff(dt.add(weeks=1).subtract(days=1)).in_weeks() == 0


def test_diff_for_humans_now_and_day(today):
    assert today.subtract(days=1).diff_for_humans() == "1 day ago"


def test_diff_for_humans_now_and_days(today):
    assert today.subtract(days=2).diff_for_humans() == "2 days ago"


def test_diff_for_humans_now_and_nearly_week(today):
    assert today.subtract(days=6).diff_for_humans() == "6 days ago"


def test_diff_for_humans_now_and_week(today):
    assert today.subtract(weeks=1).diff_for_humans() == "1 week ago"


def test_diff_for_humans_now_and_weeks(today):
    assert today.subtract(weeks=2).diff_for_humans() == "2 weeks ago"


def test_diff_for_humans_now_and_nearly_month(today):
    assert today.subtract(weeks=3).diff_for_humans() == "3 weeks ago"


def test_diff_for_humans_now_and_month():
    with pendulum.travel_to(pendulum.datetime(2016, 4, 1)):
        today = pendulum.today().date()

        assert today.subtract(weeks=4).diff_for_humans() == "4 weeks ago"
        assert today.subtract(months=1).diff_for_humans() == "1 month ago"

    with pendulum.travel_to(pendulum.datetime(2017, 3, 1)):
        today = pendulum.today().date()

        assert today.subtract(weeks=4).diff_for_humans() == "1 month ago"


def test_diff_for_humans_now_and_months(today):
    assert today.subtract(months=2).diff_for_humans() == "2 months ago"


def test_diff_for_humans_now_and_nearly_year(today):
    assert today.subtract(months=11).diff_for_humans() == "11 months ago"


def test_diff_for_humans_now_and_year(today):
    assert today.subtract(years=1).diff_for_humans() == "1 year ago"


def test_diff_for_humans_now_and_years(today):
    assert today.subtract(years=2).diff_for_humans() == "2 years ago"


def test_diff_for_humans_now_and_future_day(today):
    assert today.add(days=1).diff_for_humans() == "in 1 day"


def test_diff_for_humans_now_and_future_days(today):
    assert today.add(days=2).diff_for_humans() == "in 2 days"


def test_diff_for_humans_now_and_nearly_future_week(today):
    assert today.add(days=6).diff_for_humans() == "in 6 days"


def test_diff_for_humans_now_and_future_week(today):
    assert today.add(weeks=1).diff_for_humans() == "in 1 week"


def test_diff_for_humans_now_and_future_weeks(today):
    assert today.add(weeks=2).diff_for_humans() == "in 2 weeks"


def test_diff_for_humans_now_and_nearly_future_month(today):
    assert today.add(weeks=3).diff_for_humans() == "in 3 weeks"


def test_diff_for_humans_now_and_future_month():
    with pendulum.travel_to(pendulum.datetime(2016, 3, 1)):
        today = pendulum.today("UTC").date()

        assert today.add(weeks=4).diff_for_humans() == "in 4 weeks"
        assert today.add(months=1).diff_for_humans() == "in 1 month"

    with pendulum.travel_to(pendulum.datetime(2017, 3, 31)):
        today = pendulum.today("UTC").date()

        assert today.add(months=1).diff_for_humans() == "in 1 month"

    with pendulum.travel_to(pendulum.datetime(2017, 4, 30)):
        today = pendulum.today("UTC").date()

        assert today.add(months=1).diff_for_humans() == "in 1 month"

    with pendulum.travel_to(pendulum.datetime(2017, 1, 31)):
        today = pendulum.today("UTC").date()

        assert today.add(weeks=4).diff_for_humans() == "in 1 month"


def test_diff_for_humans_now_and_future_months(today):
    assert today.add(months=2).diff_for_humans() == "in 2 months"


def test_diff_for_humans_now_and_nearly_future_year(today):
    assert today.add(months=11).diff_for_humans() == "in 11 months"


def test_diff_for_humans_now_and_future_year(today):
    assert today.add(years=1).diff_for_humans() == "in 1 year"


def test_diff_for_humans_now_and_future_years(today):
    assert today.add(years=2).diff_for_humans() == "in 2 years"


def test_diff_for_humans_other_and_day(today):
    assert today.diff_for_humans(today.add(days=1)) == "1 day before"


def test_diff_for_humans_other_and_days(today):
    assert today.diff_for_humans(today.add(days=2)) == "2 days before"


def test_diff_for_humans_other_and_nearly_week(today):
    assert today.diff_for_humans(today.add(days=6)) == "6 days before"


def test_diff_for_humans_other_and_week(today):
    assert today.diff_for_humans(today.add(weeks=1)) == "1 week before"


def test_diff_for_humans_other_and_weeks(today):
    assert today.diff_for_humans(today.add(weeks=2)) == "2 weeks before"


def test_diff_for_humans_other_and_nearly_month(today):
    assert today.diff_for_humans(today.add(weeks=3)) == "3 weeks before"


def test_diff_for_humans_other_and_month():
    with pendulum.travel_to(pendulum.datetime(2016, 3, 1)):
        today = pendulum.today().date()

        assert today.diff_for_humans(today.add(weeks=4)) == "4 weeks before"
        assert today.diff_for_humans(today.add(months=1)) == "1 month before"

    with pendulum.travel_to(pendulum.datetime(2017, 3, 31)):
        today = pendulum.today().date()

        assert today.diff_for_humans(today.add(months=1)) == "1 month before"

    with pendulum.travel_to(pendulum.datetime(2017, 4, 30)):
        today = pendulum.today().date()

        assert today.diff_for_humans(today.add(months=1)) == "1 month before"

    with pendulum.travel_to(pendulum.datetime(2017, 1, 31)):
        today = pendulum.today().date()

        assert today.diff_for_humans(today.add(weeks=4)) == "1 month before"


def test_diff_for_humans_other_and_months(today):
    assert today.diff_for_humans(today.add(months=2)) == "2 months before"


def test_diff_for_humans_other_and_nearly_year(today):
    assert today.diff_for_humans(today.add(months=11)) == "11 months before"


def test_diff_for_humans_other_and_year(today):
    assert today.diff_for_humans(today.add(years=1)) == "1 year before"


def test_diff_for_humans_other_and_years(today):
    assert today.diff_for_humans(today.add(years=2)) == "2 years before"


def test_diff_for_humans_other_and_future_day(today):
    assert today.diff_for_humans(today.subtract(days=1)) == "1 day after"


def test_diff_for_humans_other_and_future_days(today):
    assert today.diff_for_humans(today.subtract(days=2)) == "2 days after"


def test_diff_for_humans_other_and_nearly_future_week(today):
    assert today.diff_for_humans(today.subtract(days=6)) == "6 days after"


def test_diff_for_humans_other_and_future_week(today):
    assert today.diff_for_humans(today.subtract(weeks=1)) == "1 week after"


def test_diff_for_humans_other_and_future_weeks(today):
    assert today.diff_for_humans(today.subtract(weeks=2)) == "2 weeks after"


def test_diff_for_humans_other_and_nearly_future_month(today):
    assert today.diff_for_humans(today.subtract(weeks=3)) == "3 weeks after"


def test_diff_for_humans_other_and_future_month():
    with pendulum.travel_to(pendulum.datetime(2016, 3, 1)):
        today = pendulum.today().date()

        assert today.diff_for_humans(today.subtract(weeks=4)) == "4 weeks after"
        assert today.diff_for_humans(today.subtract(months=1)) == "1 month after"

    with pendulum.travel_to(pendulum.datetime(2017, 2, 28)):
        today = pendulum.today().date()

        assert today.diff_for_humans(today.subtract(weeks=4)) == "1 month after"


def test_diff_for_humans_other_and_future_months(today):
    assert today.diff_for_humans(today.subtract(months=2)) == "2 months after"


def test_diff_for_humans_other_and_nearly_future_year(today):
    assert today.diff_for_humans(today.subtract(months=11)) == "11 months after"


def test_diff_for_humans_other_and_future_year(today):
    assert today.diff_for_humans(today.subtract(years=1)) == "1 year after"


def test_diff_for_humans_other_and_future_years(today):
    assert today.diff_for_humans(today.subtract(years=2)) == "2 years after"


def test_diff_for_humans_absolute_days(today):
    assert today.diff_for_humans(today.subtract(days=2), True) == "2 days"
    assert today.diff_for_humans(today.add(days=2), True) == "2 days"


def test_diff_for_humans_absolute_weeks(today):
    assert today.diff_for_humans(today.subtract(weeks=2), True) == "2 weeks"
    assert today.diff_for_humans(today.add(weeks=2), True) == "2 weeks"


def test_diff_for_humans_absolute_months(today):
    assert today.diff_for_humans(today.subtract(months=2), True) == "2 months"
    assert today.diff_for_humans(today.add(months=2), True) == "2 months"


def test_diff_for_humans_absolute_years(today):
    assert today.diff_for_humans(today.subtract(years=1), True) == "1 year"
    assert today.diff_for_humans(today.add(years=1), True) == "1 year"


def test_subtraction():
    d = pendulum.date(2016, 7, 5)
    future_dt = date(2016, 7, 6)
    future = d.add(days=1)

    assert (future - d).total_seconds() == 86400
    assert (future_dt - d).total_seconds() == 86400



from __future__ import annotations

import pendulum


locale = "id"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "beberapa detik yang lalu"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "beberapa detik yang lalu"

    d = pendulum.now().subtract(seconds=21)
    assert d.diff_for_humans(locale=locale) == "21 detik yang lalu"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "1 menit yang lalu"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "2 menit yang lalu"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "1 jam yang lalu"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "2 jam yang lalu"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "1 hari yang lalu"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "2 hari yang lalu"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "1 minggu yang lalu"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "2 minggu yang lalu"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "1 bulan yang lalu"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "2 bulan yang lalu"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "1 tahun yang lalu"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "2 tahun yang lalu"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "dalam beberapa detik"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "beberapa detik kemudian"
    assert d2.diff_for_humans(d, locale=locale) == "beberapa detik yang lalu"

    assert d.diff_for_humans(d2, True, locale=locale) == "beberapa detik"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "beberapa detik"



from __future__ import annotations

from datetime import timedelta

import pytest

import pendulum

from tests.conftest import assert_datetime


def test_add_years_positive():
    assert pendulum.datetime(1975, 1, 1).add(years=1).year == 1976


def test_add_years_zero():
    assert pendulum.datetime(1975, 1, 1).add(years=0).year == 1975


def test_add_years_negative():
    assert pendulum.datetime(1975, 1, 1).add(years=-1).year == 1974


def test_add_months_positive():
    assert pendulum.datetime(1975, 12, 1).add(months=1).month == 1


def test_add_months_zero():
    assert pendulum.datetime(1975, 12, 1).add(months=0).month == 12


def test_add_months_negative():
    assert pendulum.datetime(1975, 12, 1).add(months=-1).month == 11


def test_add_month_with_overflow():
    assert pendulum.datetime(2012, 1, 31).add(months=1).month == 2


def test_add_days_positive():
    assert pendulum.datetime(1975, 5, 31).add(days=1).day == 1


def test_add_days_zero():
    assert pendulum.datetime(1975, 5, 31).add(days=0).day == 31


def test_add_days_negative():
    assert pendulum.datetime(1975, 5, 31).add(days=-1).day == 30


def test_add_weeks_positive():
    assert pendulum.datetime(1975, 5, 21).add(weeks=1).day == 28


def test_add_weeks_zero():
    assert pendulum.datetime(1975, 5, 21).add(weeks=0).day == 21


def test_add_weeks_negative():
    assert pendulum.datetime(1975, 5, 21).add(weeks=-1).day == 14


def test_add_hours_positive():
    assert pendulum.datetime(1975, 5, 21, 0, 0, 0).add(hours=1).hour == 1


def test_add_hours_zero():
    assert pendulum.datetime(1975, 5, 21, 0, 0, 0).add(hours=0).hour == 0


def test_add_hours_negative():
    assert pendulum.datetime(1975, 5, 21, 0, 0, 0).add(hours=-1).hour == 23


def test_add_minutes_positive():
    assert pendulum.datetime(1975, 5, 21, 0, 0, 0).add(minutes=1).minute == 1


def test_add_minutes_zero():
    assert pendulum.datetime(1975, 5, 21, 0, 0, 0).add(minutes=0).minute == 0


def test_add_minutes_negative():
    assert pendulum.datetime(1975, 5, 21, 0, 0, 0).add(minutes=-1).minute == 59


def test_add_seconds_positive():
    assert pendulum.datetime(1975, 5, 21, 0, 0, 0).add(seconds=1).second == 1


def test_add_seconds_zero():
    assert pendulum.datetime(1975, 5, 21, 0, 0, 0).add(seconds=0).second == 0


def test_add_seconds_negative():
    assert pendulum.datetime(1975, 5, 21, 0, 0, 0).add(seconds=-1).second == 59


def test_add_timedelta():
    delta = timedelta(days=6, seconds=45, microseconds=123456)
    d = pendulum.datetime(2015, 3, 14, 3, 12, 15, 654321)

    d = d + delta
    assert d.day == 20
    assert d.minute == 13
    assert d.second == 0
    assert d.microsecond == 777777


def test_add_duration():
    duration = pendulum.duration(
        years=2, months=3, days=6, seconds=45, microseconds=123456
    )
    d = pendulum.datetime(2015, 3, 14, 3, 12, 15, 654321)

    d = d + duration
    assert d.year == 2017
    assert d.month == 6
    assert d.day == 20
    assert d.hour == 3
    assert d.minute == 13
    assert d.second == 0
    assert d.microsecond == 777777


def test_addition_invalid_type():
    d = pendulum.datetime(2015, 3, 14, 3, 12, 15, 654321)

    with pytest.raises(TypeError):
        d + 3

    with pytest.raises(TypeError):
        3 + d


def test_add_to_fixed_timezones():
    dt = pendulum.parse("2015-03-08T01:00:00-06:00")
    dt = dt.add(weeks=1)
    dt = dt.add(hours=1)

    assert_datetime(dt, 2015, 3, 15, 2, 0, 0)
    assert dt.timezone_name == "-06:00"
    assert dt.offset == -6 * 3600


def test_add_time_to_new_transition_skipped():
    dt = pendulum.datetime(2013, 3, 31, 1, 59, 59, 999999, tz="Europe/Paris")

    assert_datetime(dt, 2013, 3, 31, 1, 59, 59, 999999)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.offset == 3600
    assert not dt.is_dst()

    dt = dt.add(microseconds=1)

    assert_datetime(dt, 2013, 3, 31, 3, 0, 0, 0)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.offset == 7200
    assert dt.is_dst()

    dt = pendulum.datetime(2013, 3, 10, 1, 59, 59, 999999, tz="America/New_York")

    assert_datetime(dt, 2013, 3, 10, 1, 59, 59, 999999)
    assert dt.timezone_name == "America/New_York"
    assert dt.offset == -5 * 3600
    assert not dt.is_dst()

    dt = dt.add(microseconds=1)

    assert_datetime(dt, 2013, 3, 10, 3, 0, 0, 0)
    assert dt.timezone_name == "America/New_York"
    assert dt.offset == -4 * 3600
    assert dt.is_dst()

    dt = pendulum.datetime(1957, 4, 28, 1, 59, 59, 999999, tz="America/New_York")

    assert_datetime(dt, 1957, 4, 28, 1, 59, 59, 999999)
    assert dt.timezone_name == "America/New_York"
    assert dt.offset == -5 * 3600
    assert not dt.is_dst()

    dt = dt.add(microseconds=1)

    assert_datetime(dt, 1957, 4, 28, 3, 0, 0, 0)
    assert dt.timezone_name == "America/New_York"
    assert dt.offset == -4 * 3600
    assert dt.is_dst()


def test_add_time_to_new_transition_skipped_big():
    dt = pendulum.datetime(2013, 3, 31, 1, tz="Europe/Paris")

    assert_datetime(dt, 2013, 3, 31, 1, 0, 0, 0)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.offset == 3600
    assert not dt.is_dst()

    dt = dt.add(weeks=1)

    assert_datetime(dt, 2013, 4, 7, 1, 0, 0, 0)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.offset == 7200
    assert dt.is_dst()


def test_add_time_to_new_transition_repeated():
    dt = pendulum.datetime(2013, 10, 27, 1, 59, 59, 999999, tz="Europe/Paris")
    dt = dt.add(hours=1)

    assert_datetime(dt, 2013, 10, 27, 2, 59, 59, 999999)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.offset == 7200
    assert dt.is_dst()

    dt = dt.add(microseconds=1)

    assert_datetime(dt, 2013, 10, 27, 2, 0, 0, 0)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.offset == 3600
    assert not dt.is_dst()

    dt = pendulum.datetime(2013, 11, 3, 0, 59, 59, 999999, tz="America/New_York")
    dt = dt.add(hours=1)

    assert_datetime(dt, 2013, 11, 3, 1, 59, 59, 999999)
    assert dt.timezone_name == "America/New_York"
    assert dt.offset == -4 * 3600
    assert dt.is_dst()

    dt = dt.add(microseconds=1)

    assert_datetime(dt, 2013, 11, 3, 1, 0, 0, 0)
    assert dt.timezone_name == "America/New_York"
    assert dt.offset == -5 * 3600
    assert not dt.is_dst()


def test_add_time_to_new_transition_repeated_big():
    dt = pendulum.datetime(2013, 10, 27, 1, tz="Europe/Paris")

    assert_datetime(dt, 2013, 10, 27, 1, 0, 0, 0)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.offset == 7200
    assert dt.is_dst()

    dt = dt.add(weeks=1)

    assert_datetime(dt, 2013, 11, 3, 1, 0, 0, 0)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.offset == 3600
    assert not dt.is_dst()


def test_add_duration_across_transition():
    dt = pendulum.datetime(2017, 3, 11, 10, 45, tz="America/Los_Angeles")
    new = dt + pendulum.duration(hours=24)

    assert_datetime(new, 2017, 3, 12, 11, 45)


def test_add_duration_across_transition_days():
    dt = pendulum.datetime(2017, 3, 11, 10, 45, tz="America/Los_Angeles")
    new = dt + pendulum.duration(days=1)

    assert_datetime(new, 2017, 3, 12, 10, 45)

    dt = pendulum.datetime(2023, 11, 5, 0, 0, tz="America/Chicago")
    new = dt + pendulum.duration(days=1)

    assert_datetime(new, 2023, 11, 6, 0, 0)


def test_interval_over_midnight_tz():
    start = pendulum.datetime(2018, 2, 25, tz="Europe/Paris")
    end = start.add(hours=1)
    interval = end - start
    new_end = start + interval

    assert new_end == end



from __future__ import annotations

import pendulum


def test_in_weeks():
    it = pendulum.duration(days=17)
    assert it.in_weeks() == 2


def test_in_days():
    it = pendulum.duration(days=3)
    assert it.in_days() == 3


def test_in_hours():
    it = pendulum.duration(days=3, minutes=72)
    assert it.in_hours() == 73


def test_in_minutes():
    it = pendulum.duration(minutes=6, seconds=72)
    assert it.in_minutes() == 7


def test_in_seconds():
    it = pendulum.duration(seconds=72)
    assert it.in_seconds() == 72



from __future__ import annotations

import pendulum


locale = "de"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "vor 1 Sekunde"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "vor 2 Sekunden"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "vor 1 Minute"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "vor 2 Minuten"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "vor 1 Stunde"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "vor 2 Stunden"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "vor 1 Tag"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "vor 2 Tagen"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "vor 1 Woche"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "vor 2 Wochen"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "vor 1 Monat"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "vor 2 Monaten"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "vor 1 Jahr"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "vor 2 Jahren"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "in 1 Sekunde"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "1 Sekunde später"
    assert d2.diff_for_humans(d, locale=locale) == "1 Sekunde zuvor"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 Sekunde"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 Sekunden"



from __future__ import annotations

from datetime import time

import pendulum

from tests.conftest import assert_time


def test_equal_to_true():
    t1 = pendulum.time(1, 2, 3)
    t2 = pendulum.time(1, 2, 3)
    t3 = time(1, 2, 3)

    assert t1 == t2
    assert t1 == t3


def test_equal_to_false():
    t1 = pendulum.time(1, 2, 3)
    t2 = pendulum.time(1, 2, 4)
    t3 = time(1, 2, 4)

    assert t1 != t2
    assert t1 != t3


def test_not_equal_to_none():
    t1 = pendulum.time(1, 2, 3)

    assert t1 is not None


def test_greater_than_true():
    t1 = pendulum.time(1, 2, 3)
    t2 = pendulum.time(1, 2, 2)
    t3 = time(1, 2, 2)

    assert t1 > t2
    assert t1 > t3


def test_greater_than_false():
    t1 = pendulum.time(1, 2, 2)
    t2 = pendulum.time(1, 2, 3)
    t3 = time(1, 2, 3)

    assert not t1 > t2
    assert not t1 > t3


def test_greater_than_or_equal_true():
    t1 = pendulum.time(1, 2, 3)
    t2 = pendulum.time(1, 2, 2)
    t3 = time(1, 2, 2)

    assert t1 >= t2
    assert t1 >= t3


def test_greater_than_or_equal_true_equal():
    t1 = pendulum.time(1, 2, 3)
    t2 = pendulum.time(1, 2, 3)
    t3 = time(1, 2, 3)

    assert t1 >= t2
    assert t1 >= t3


def test_greater_than_or_equal_false():
    t1 = pendulum.time(1, 2, 2)
    t2 = pendulum.time(1, 2, 3)
    t3 = time(1, 2, 3)

    assert not t1 >= t2
    assert not t1 >= t3


def test_less_than_true():
    t1 = pendulum.time(1, 2, 2)
    t2 = pendulum.time(1, 2, 3)
    t3 = time(1, 2, 3)

    assert t1 < t2
    assert t1 < t3


def test_less_than_false():
    t1 = pendulum.time(1, 2, 3)
    t2 = pendulum.time(1, 2, 2)
    t3 = time(1, 2, 2)

    assert not t1 < t2
    assert not t1 < t3


def test_less_than_or_equal_true():
    t1 = pendulum.time(1, 2, 2)
    t2 = pendulum.time(1, 2, 3)
    t3 = time(1, 2, 3)

    assert t1 <= t2
    assert t1 <= t3


def test_less_than_or_equal_true_equal():
    t1 = pendulum.time(1, 2, 3)
    t2 = pendulum.time(1, 2, 3)
    t3 = time(1, 2, 3)

    assert t1 <= t2
    assert t1 <= t3


def test_less_than_or_equal_false():
    t1 = pendulum.time(1, 2, 3)
    t2 = pendulum.time(1, 2, 2)
    t3 = time(1, 2, 2)

    assert not t1 <= t2
    assert not t1 <= t3


def test_closest():
    instance = pendulum.time(12, 34, 56)
    t1 = pendulum.time(12, 34, 54)
    t2 = pendulum.time(12, 34, 59)
    closest = instance.closest(t1, t2)
    assert t1 == closest

    closest = instance.closest(t2, t1)
    assert t1 == closest


def test_closest_with_time():
    instance = pendulum.time(12, 34, 56)
    t1 = pendulum.time(12, 34, 54)
    t2 = pendulum.time(12, 34, 59)
    closest = instance.closest(t1, t2)

    assert_time(closest, 12, 34, 54)


def test_closest_with_equals():
    instance = pendulum.time(12, 34, 56)
    t1 = pendulum.time(12, 34, 56)
    t2 = pendulum.time(12, 34, 59)
    closest = instance.closest(t1, t2)
    assert t1 == closest


def test_farthest():
    instance = pendulum.time(12, 34, 56)
    t1 = pendulum.time(12, 34, 54)
    t2 = pendulum.time(12, 34, 59)
    farthest = instance.farthest(t1, t2)
    assert t2 == farthest

    farthest = instance.farthest(t2, t1)
    assert t2 == farthest


def test_farthest_with_time():
    instance = pendulum.time(12, 34, 56)
    t1 = pendulum.time(12, 34, 54)
    t2 = pendulum.time(12, 34, 59)
    farthest = instance.farthest(t1, t2)

    assert_time(farthest, 12, 34, 59)


def test_farthest_with_equals():
    instance = pendulum.time(12, 34, 56)
    t1 = pendulum.time(12, 34, 56)
    t2 = pendulum.time(12, 34, 59)

    farthest = instance.farthest(t1, t2)
    assert t2 == farthest


def test_comparison_to_unsupported():
    t1 = pendulum.now().time()

    assert t1 != "test"
    assert t1 not in ["test"]



from __future__ import annotations

from functools import cache
from zoneinfo import available_timezones

from pendulum.tz.local_timezone import get_local_timezone
from pendulum.tz.local_timezone import set_local_timezone
from pendulum.tz.local_timezone import test_local_timezone
from pendulum.tz.timezone import UTC
from pendulum.tz.timezone import FixedTimezone
from pendulum.tz.timezone import Timezone


PRE_TRANSITION = "pre"
POST_TRANSITION = "post"
TRANSITION_ERROR = "error"

_tz_cache: dict[int, FixedTimezone] = {}


@cache
def timezones() -> set[str]:
    return available_timezones()


def fixed_timezone(offset: int) -> FixedTimezone:
    """
    Return a Timezone instance given its offset in seconds.
    """
    if offset in _tz_cache:
        return _tz_cache[offset]

    tz = FixedTimezone(offset)
    _tz_cache[offset] = tz

    return tz


def local_timezone() -> Timezone | FixedTimezone:
    """
    Return the local timezone.
    """
    return get_local_timezone()


__all__ = [
    "UTC",
    "FixedTimezone",
    "Timezone",
    "fixed_timezone",
    "get_local_timezone",
    "local_timezone",
    "set_local_timezone",
    "test_local_timezone",
    "timezones",
]



from __future__ import annotations

import pendulum

from tests.conftest import assert_duration


def test_multiply():
    it = pendulum.duration(days=6, seconds=34, microseconds=522222)
    mul = it * 2

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 0, 0, 1, 5, 0, 1, 9, 44444)

    it = pendulum.duration(days=6, seconds=34, microseconds=522222)
    mul = 2 * it

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 0, 0, 1, 5, 0, 1, 9, 44444)

    it = pendulum.duration(
        years=2, months=3, weeks=4, days=6, seconds=34, microseconds=522222
    )
    mul = 2 * it

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 4, 6, 9, 5, 0, 1, 9, 44444)


def test_divide():
    it = pendulum.duration(days=2, seconds=34, microseconds=522222)
    mul = it / 2

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 0, 0, 0, 1, 0, 0, 17, 261111)

    it = pendulum.duration(days=2, seconds=35, microseconds=522222)
    mul = it / 2

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 0, 0, 0, 1, 0, 0, 17, 761111)

    it = pendulum.duration(days=2, seconds=35, microseconds=522222)
    mul = it / 1.1

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 0, 0, 0, 1, 19, 38, 43, 202020)

    it = pendulum.duration(years=2, months=4, days=2, seconds=35, microseconds=522222)
    mul = it / 2

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 1, 2, 0, 1, 0, 0, 17, 761111)

    it = pendulum.duration(years=2, months=4, days=2, seconds=35, microseconds=522222)
    mul = it / 2.0

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 1, 2, 0, 1, 0, 0, 17, 761111)


def test_floor_divide():
    it = pendulum.duration(days=2, seconds=34, microseconds=522222)
    mul = it // 2

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 0, 0, 0, 1, 0, 0, 17, 261111)

    it = pendulum.duration(days=2, seconds=35, microseconds=522222)
    mul = it // 3

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 0, 0, 0, 0, 16, 0, 11, 840740)

    it = pendulum.duration(years=2, months=4, days=2, seconds=34, microseconds=522222)
    mul = it // 2

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 1, 2, 0, 1, 0, 0, 17, 261111)

    it = pendulum.duration(years=2, months=4, days=2, seconds=35, microseconds=522222)
    mul = it // 3

    assert isinstance(mul, pendulum.Duration)
    assert_duration(mul, 0, 1, 0, 0, 16, 0, 11, 840740)



from __future__ import annotations

import zoneinfo

from datetime import datetime
from datetime import timedelta

import pytest

import pendulum

from pendulum import timezone
from pendulum.tz import fixed_timezone
from pendulum.tz.exceptions import AmbiguousTime
from pendulum.tz.exceptions import NonExistingTime
from tests.conftest import assert_datetime


@pytest.fixture(autouse=True)
def setup():
    pendulum.tz._tz_cache = {}

    yield

    pendulum.tz._tz_cache = {}


def test_basic_convert():
    dt = datetime(2016, 6, 1, 12, 34, 56, 123456, fold=1)
    tz = timezone("Europe/Paris")
    dt = tz.convert(dt)

    assert dt.year == 2016
    assert dt.month == 6
    assert dt.day == 1
    assert dt.hour == 12
    assert dt.minute == 34
    assert dt.second == 56
    assert dt.microsecond == 123456
    assert dt.tzinfo.name == "Europe/Paris"
    assert dt.tzinfo.utcoffset(dt) == timedelta(seconds=7200)
    assert dt.tzinfo.dst(dt) == timedelta(seconds=3600)


def test_equality():
    assert timezone("Europe/Paris") == timezone("Europe/Paris")
    assert timezone("Europe/Paris") != timezone("Europe/Berlin")


def test_skipped_time_with_pre_rule():
    dt = datetime(2013, 3, 31, 2, 30, 45, 123456, fold=0)
    tz = timezone("Europe/Paris")
    dt = tz.convert(dt)

    assert dt.year == 2013
    assert dt.month == 3
    assert dt.day == 31
    assert dt.hour == 1
    assert dt.minute == 30
    assert dt.second == 45
    assert dt.microsecond == 123456
    assert dt.tzinfo.name == "Europe/Paris"
    assert dt.tzinfo.utcoffset(dt) == timedelta(seconds=3600)
    assert dt.tzinfo.dst(dt) == timedelta()


def test_skipped_time_with_post_rule():
    dt = datetime(2013, 3, 31, 2, 30, 45, 123456, fold=1)
    tz = timezone("Europe/Paris")
    dt = tz.convert(dt)

    assert dt.year == 2013
    assert dt.month == 3
    assert dt.day == 31
    assert dt.hour == 3
    assert dt.minute == 30
    assert dt.second == 45
    assert dt.microsecond == 123456
    assert dt.tzinfo.name == "Europe/Paris"
    assert dt.tzinfo.utcoffset(dt) == timedelta(seconds=7200)
    assert dt.tzinfo.dst(dt) == timedelta(seconds=3600)


def test_skipped_time_with_error():
    dt = datetime(2013, 3, 31, 2, 30, 45, 123456)
    tz = timezone("Europe/Paris")
    with pytest.raises(NonExistingTime):
        tz.convert(dt, raise_on_unknown_times=True)


def test_repeated_time():
    dt = datetime(2013, 10, 27, 2, 30, 45, 123456, fold=1)
    tz = timezone("Europe/Paris")
    dt = tz.convert(dt)

    assert dt.year == 2013
    assert dt.month == 10
    assert dt.day == 27
    assert dt.hour == 2
    assert dt.minute == 30
    assert dt.second == 45
    assert dt.microsecond == 123456
    assert dt.tzinfo.name == "Europe/Paris"
    assert dt.tzinfo.utcoffset(dt) == timedelta(seconds=3600)
    assert dt.tzinfo.dst(dt) == timedelta()


def test_repeated_time_pre_rule():
    dt = datetime(2013, 10, 27, 2, 30, 45, 123456, fold=0)
    tz = timezone("Europe/Paris")
    dt = tz.convert(dt)

    assert dt.year == 2013
    assert dt.month == 10
    assert dt.day == 27
    assert dt.hour == 2
    assert dt.minute == 30
    assert dt.second == 45
    assert dt.microsecond == 123456
    assert dt.tzinfo.name == "Europe/Paris"
    assert dt.tzinfo.utcoffset(dt) == timedelta(seconds=7200)
    assert dt.tzinfo.dst(dt) == timedelta(seconds=3600)


def test_repeated_time_with_error():
    dt = datetime(2013, 10, 27, 2, 30, 45, 123456)
    tz = timezone("Europe/Paris")
    with pytest.raises(AmbiguousTime):
        tz.convert(dt, raise_on_unknown_times=True)


def test_pendulum_create_basic():
    dt = pendulum.datetime(2016, 6, 1, 12, 34, 56, 123456, tz="Europe/Paris")

    assert_datetime(dt, 2016, 6, 1, 12, 34, 56, 123456)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.offset == 7200
    assert dt.is_dst()


def test_pendulum_create_skipped():
    dt = pendulum.datetime(2013, 3, 31, 2, 30, 45, 123456, tz="Europe/Paris")

    assert isinstance(dt, pendulum.DateTime)
    assert_datetime(dt, 2013, 3, 31, 3, 30, 45, 123456)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.tzinfo.utcoffset(dt) == timedelta(seconds=7200)
    assert dt.tzinfo.dst(dt) == timedelta(seconds=3600)


def test_pendulum_create_skipped_with_pre_rule():
    dt = pendulum.datetime(2013, 3, 31, 2, 30, 45, 123456, tz="Europe/Paris", fold=0)

    assert_datetime(dt, 2013, 3, 31, 1, 30, 45, 123456)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.tzinfo.utcoffset(dt) == timedelta(seconds=3600)
    assert dt.tzinfo.dst(dt) == timedelta()


def test_pendulum_create_skipped_with_error():
    with pytest.raises(NonExistingTime):
        pendulum.datetime(
            2013,
            3,
            31,
            2,
            30,
            45,
            123456,
            tz="Europe/Paris",
            raise_on_unknown_times=True,
        )


def test_pendulum_create_repeated():
    dt = pendulum.datetime(2013, 10, 27, 2, 30, 45, 123456, tz="Europe/Paris")

    assert_datetime(dt, 2013, 10, 27, 2, 30, 45, 123456)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.tzinfo.utcoffset(dt) == timedelta(seconds=3600)
    assert dt.tzinfo.dst(dt) == timedelta()


def test_pendulum_create_repeated_with_pre_rule():
    dt = pendulum.datetime(
        2013,
        10,
        27,
        2,
        30,
        45,
        123456,
        tz="Europe/Paris",
        fold=0,
    )

    assert_datetime(dt, 2013, 10, 27, 2, 30, 45, 123456)
    assert dt.timezone_name == "Europe/Paris"
    assert dt.tzinfo.utcoffset(dt) == timedelta(seconds=7200)
    assert dt.tzinfo.dst(dt) == timedelta(seconds=3600)


def test_pendulum_create_repeated_with_error():
    with pytest.raises(AmbiguousTime):
        pendulum.datetime(
            2013,
            10,
            27,
            2,
            30,
            45,
            123456,
            tz="Europe/Paris",
            raise_on_unknown_times=True,
        )


def test_convert_accept_pendulum_instance():
    dt = pendulum.datetime(2016, 8, 7, 12, 53, 54)
    tz = timezone("Europe/Paris")
    new = tz.convert(dt)

    assert isinstance(new, pendulum.DateTime)
    assert_datetime(new, 2016, 8, 7, 14, 53, 54)


def test_utcoffset():
    tz = pendulum.timezone("America/Guayaquil")
    utcoffset = tz.utcoffset(pendulum.now("UTC"))
    assert utcoffset == timedelta(0, -18000)


def test_utcoffset_pre_transition():
    tz = pendulum.timezone("America/Chicago")
    utcoffset = tz.utcoffset(datetime(1883, 11, 18))
    assert utcoffset == timedelta(days=-1, seconds=65364)


def test_dst():
    tz = pendulum.timezone("Europe/Amsterdam")
    dst = tz.dst(datetime(1940, 7, 1))
    native_tz = zoneinfo.ZoneInfo("Europe/Amsterdam")

    assert dst == native_tz.dst(datetime(1940, 7, 1))


def test_short_timezones_should_not_modify_time():
    tz = pendulum.timezone("EST")
    dt = tz.datetime(2017, 6, 15, 14, 0, 0)

    assert dt.year == 2017
    assert dt.month == 6
    assert dt.day == 15
    assert dt.hour == 14
    assert dt.minute == 0
    assert dt.second == 0

    tz = pendulum.timezone("HST")
    dt = tz.datetime(2017, 6, 15, 14, 0, 0)

    assert dt.year == 2017
    assert dt.month == 6
    assert dt.day == 15
    assert dt.hour == 14
    assert dt.minute == 0
    assert dt.second == 0


def test_after_last_transition():
    tz = pendulum.timezone("Europe/Paris")
    dt = tz.datetime(2135, 6, 15, 14, 0, 0)

    assert dt.year == 2135
    assert dt.month == 6
    assert dt.day == 15
    assert dt.hour == 14
    assert dt.minute == 0
    assert dt.second == 0
    assert dt.microsecond == 0


@pytest.mark.skip(
    reason=(
        "zoneinfo does not currently support POSIX transition"
        " rules to go beyond the last fixed transition."
    )
)
def test_on_last_transition():
    tz = pendulum.timezone("Europe/Paris")
    dt = pendulum.naive(2037, 10, 25, 2, 30)
    dt = tz.convert(dt, dst_rule=pendulum.POST_TRANSITION)

    assert dt.year == 2037
    assert dt.month == 10
    assert dt.day == 25
    assert dt.hour == 2
    assert dt.minute == 30
    assert dt.second == 0
    assert dt.microsecond == 0
    assert dt.utcoffset().total_seconds() == 3600

    dt = pendulum.naive(2037, 10, 25, 2, 30)
    dt = tz.convert(dt, dst_rule=pendulum.PRE_TRANSITION)

    assert dt.year == 2037
    assert dt.month == 10
    assert dt.day == 25
    assert dt.hour == 2
    assert dt.minute == 30
    assert dt.second == 0
    assert dt.microsecond == 0
    assert dt.utcoffset().total_seconds() == 7200


def test_convert_fold_attribute_is_honored():
    tz = pendulum.timezone("US/Eastern")
    dt = datetime(2014, 11, 2, 1, 30)

    new = tz.convert(dt)
    assert new.strftime("%z") == "-0400"

    new = tz.convert(dt.replace(fold=1))
    assert new.strftime("%z") == "-0500"


def test_utcoffset_fold_attribute_is_honored():
    tz = pendulum.timezone("US/Eastern")
    dt = datetime(2014, 11, 2, 1, 30)

    offset = tz.utcoffset(dt)

    assert offset.total_seconds() == -4 * 3600

    offset = tz.utcoffset(dt.replace(fold=1))

    assert offset.total_seconds() == -5 * 3600


def test_dst_fold_attribute_is_honored():
    tz = pendulum.timezone("US/Eastern")
    dt = datetime(2014, 11, 2, 1, 30)

    offset = tz.dst(dt)

    assert offset.total_seconds() == 3600

    offset = tz.dst(dt.replace(fold=1))

    assert offset.total_seconds() == 0


def test_tzname_fold_attribute_is_honored():
    tz = pendulum.timezone("US/Eastern")
    dt = datetime(2014, 11, 2, 1, 30)

    name = tz.tzname(dt)

    assert name == "EDT"

    name = tz.tzname(dt.replace(fold=1))

    assert name == "EST"


def test_constructor_fold_attribute_is_honored():
    tz = pendulum.timezone("US/Eastern")
    dt = datetime(2014, 11, 2, 1, 30, tzinfo=tz)

    assert dt.strftime("%z") == "-0400"

    dt = datetime(2014, 11, 2, 1, 30, tzinfo=tz, fold=1)

    assert dt.strftime("%z") == "-0500"


def test_datetime():
    tz = timezone("Europe/Paris")

    dt = tz.datetime(2013, 3, 24, 1, 30)
    assert dt.year == 2013
    assert dt.month == 3
    assert dt.day == 24
    assert dt.hour == 1
    assert dt.minute == 30
    assert dt.second == 0
    assert dt.microsecond == 0

    dt = tz.datetime(2013, 3, 31, 2, 30)
    assert dt.year == 2013
    assert dt.month == 3
    assert dt.day == 31
    assert dt.hour == 3
    assert dt.minute == 30
    assert dt.second == 0
    assert dt.microsecond == 0


def test_fixed_timezone():
    tz = fixed_timezone(19800)
    tz2 = fixed_timezone(18000)
    dt = datetime(2016, 11, 26, tzinfo=tz)

    assert tz2.utcoffset(dt).total_seconds() == 18000
    assert tz2.dst(dt) == timedelta()


def test_fixed_equality():
    assert fixed_timezone(19800) == fixed_timezone(19800)
    assert fixed_timezone(19800) != fixed_timezone(19801)


def test_just_before_last_transition():
    tz = pendulum.timezone("Asia/Shanghai")
    dt = datetime(1991, 4, 20, 1, 49, 8, fold=0)
    dt = tz.convert(dt)

    epoch = datetime(1970, 1, 1, tzinfo=timezone("UTC"))
    expected = (dt - epoch).total_seconds()
    assert expected == 672079748.0


@pytest.mark.skip(
    reason=(
        "zoneinfo does not currently support POSIX transition"
        " rules to go beyond the last fixed transition."
    )
)
def test_timezones_are_extended():
    tz = pendulum.timezone("Europe/Paris")
    dt = tz.convert(pendulum.naive(2134, 2, 13, 1))

    assert_datetime(dt, 2134, 2, 13, 1)
    assert dt.utcoffset().total_seconds() == 3600
    assert dt.dst() == timedelta()

    dt = tz.convert(pendulum.naive(2134, 3, 28, 2, 30))

    assert_datetime(dt, 2134, 3, 28, 3, 30)
    assert dt.utcoffset().total_seconds() == 7200
    assert dt.dst() == timedelta(seconds=3600)

    dt = tz.convert(pendulum.naive(2134, 7, 11, 2, 30))

    assert_datetime(dt, 2134, 7, 11, 2, 30)
    assert dt.utcoffset().total_seconds() == 7200
    assert dt.dst() == timedelta(seconds=3600)

    dt = tz.convert(pendulum.naive(2134, 10, 31, 2, 30, fold=0))

    assert_datetime(dt, 2134, 10, 31, 2, 30)
    assert dt.utcoffset().total_seconds() == 7200
    assert dt.dst() == timedelta(seconds=3600)

    dt = tz.convert(pendulum.naive(2134, 10, 31, 2, 30))

    assert_datetime(dt, 2134, 10, 31, 2, 30)
    assert dt.utcoffset().total_seconds() == 3600
    assert dt.dst() == timedelta()


def test_repr():
    tz = timezone("Europe/Paris")

    assert repr(tz) == "Timezone('Europe/Paris')"



from __future__ import annotations

from datetime import timedelta

import pendulum

from tests.conftest import assert_duration


def test_add_interval():
    p1 = pendulum.duration(days=23, seconds=32)
    p2 = pendulum.duration(days=12, seconds=30)

    p = p1 + p2
    assert_duration(p, 0, 0, 5, 0, 0, 1, 2)


def test_add_timedelta():
    p1 = pendulum.duration(days=23, seconds=32)
    p2 = timedelta(days=12, seconds=30)

    p = p1 + p2
    assert_duration(p, 0, 0, 5, 0, 0, 1, 2)


def test_add_unsupported():
    p = pendulum.duration(days=23, seconds=32)
    assert NotImplemented == p.__add__(5)


def test_sub_interval():
    p1 = pendulum.duration(days=23, seconds=32)
    p2 = pendulum.duration(days=12, seconds=28)

    p = p1 - p2
    assert_duration(p, 0, 0, 1, 4, 0, 0, 4)


def test_sub_timedelta():
    p1 = pendulum.duration(days=23, seconds=32)
    p2 = timedelta(days=12, seconds=28)

    p = p1 - p2
    assert_duration(p, 0, 0, 1, 4, 0, 0, 4)


def test_sub_unsupported():
    p = pendulum.duration(days=23, seconds=32)
    assert NotImplemented == p.__sub__(5)


def test_neg():
    p = pendulum.duration(days=23, seconds=32)
    assert_duration(-p, 0, 0, -3, -2, 0, 0, -32)



from __future__ import annotations

from pendulum.locales.nl.custom import translations as custom_translations


"""
nl locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "one"
    if ((n == n and (n == 1)) and (0 == 0 and (0 == 0)))
    else "other",
    "ordinal": lambda n: "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "ma",
                1: "di",
                2: "wo",
                3: "do",
                4: "vr",
                5: "za",
                6: "zo",
            },
            "narrow": {0: "M", 1: "D", 2: "W", 3: "D", 4: "V", 5: "Z", 6: "Z"},
            "short": {0: "ma", 1: "di", 2: "wo", 3: "do", 4: "vr", 5: "za", 6: "zo"},
            "wide": {
                0: "maandag",
                1: "dinsdag",
                2: "woensdag",
                3: "donderdag",
                4: "vrijdag",
                5: "zaterdag",
                6: "zondag",
            },
        },
        "months": {
            "abbreviated": {
                1: "jan.",
                2: "feb.",
                3: "mrt.",
                4: "apr.",
                5: "mei",
                6: "jun.",
                7: "jul.",
                8: "aug.",
                9: "sep.",
                10: "okt.",
                11: "nov.",
                12: "dec.",
            },
            "narrow": {
                1: "J",
                2: "F",
                3: "M",
                4: "A",
                5: "M",
                6: "J",
                7: "J",
                8: "A",
                9: "S",
                10: "O",
                11: "N",
                12: "D",
            },
            "wide": {
                1: "januari",
                2: "februari",
                3: "maart",
                4: "april",
                5: "mei",
                6: "juni",
                7: "juli",
                8: "augustus",
                9: "september",
                10: "oktober",
                11: "november",
                12: "december",
            },
        },
        "units": {
            "year": {"one": "{0} jaar", "other": "{0} jaar"},
            "month": {"one": "{0} maand", "other": "{0} maanden"},
            "week": {"one": "{0} week", "other": "{0} weken"},
            "day": {"one": "{0} dag", "other": "{0} dagen"},
            "hour": {"one": "{0} uur", "other": "{0} uur"},
            "minute": {"one": "{0} minuut", "other": "{0} minuten"},
            "second": {"one": "{0} seconde", "other": "{0} seconden"},
            "microsecond": {"one": "{0} microseconde", "other": "{0} microseconden"},
        },
        "relative": {
            "year": {
                "future": {"other": "over {0} jaar", "one": "over {0} jaar"},
                "past": {"other": "{0} jaar geleden", "one": "{0} jaar geleden"},
            },
            "month": {
                "future": {"other": "over {0} maanden", "one": "over {0} maand"},
                "past": {"other": "{0} maanden geleden", "one": "{0} maand geleden"},
            },
            "week": {
                "future": {"other": "over {0} weken", "one": "over {0} week"},
                "past": {"other": "{0} weken geleden", "one": "{0} week geleden"},
            },
            "day": {
                "future": {"other": "over {0} dagen", "one": "over {0} dag"},
                "past": {"other": "{0} dagen geleden", "one": "{0} dag geleden"},
            },
            "hour": {
                "future": {"other": "over {0} uur", "one": "over {0} uur"},
                "past": {"other": "{0} uur geleden", "one": "{0} uur geleden"},
            },
            "minute": {
                "future": {"other": "over {0} minuten", "one": "over {0} minuut"},
                "past": {"other": "{0} minuten geleden", "one": "{0} minuut geleden"},
            },
            "second": {
                "future": {"other": "over {0} seconden", "one": "over {0} seconde"},
                "past": {"other": "{0} seconden geleden", "one": "{0} seconde geleden"},
            },
        },
        "day_periods": {
            "midnight": "middernacht",
            "am": "a.m.",
            "pm": "p.m.",
            "morning1": "‘s ochtends",
            "afternoon1": "‘s middags",
            "evening1": "‘s avonds",
            "night1": "‘s nachts",
            "week_data": {
                "min_days": 1,
                "first_day": 0,
                "weekend_start": 5,
                "weekend_end": 6,
            },
        },
    },
    "custom": custom_translations,
}



"""
en-gb custom locale file.
"""
from __future__ import annotations


translations = {
    "units": {"few_second": "a few seconds"},
    # Relative time
    "ago": "{} ago",
    "from_now": "in {}",
    "after": "{0} after",
    "before": "{0} before",
    # Ordinals
    "ordinal": {"one": "st", "two": "nd", "few": "rd", "other": "th"},
    # Date formats
    "date_formats": {
        "LTS": "HH:mm:ss",
        "LT": "HH:mm",
        "L": "DD/MM/YYYY",
        "LL": "D MMMM YYYY",
        "LLL": "D MMMM YYYY HH:mm",
        "LLLL": "dddd, D MMMM YYYY HH:mm",
    },
}
